@@ -1,17 +1,25 @@
+# pylint: disable=too-many-lines
 """CmdStan-specific conversion code."""
+try:
+    import ujson as json
+except ImportError:
+    # Can't find ujson using json
+    # mypy struggles with conditional imports expressed as catching ImportError:
+    # https://github.com/python/mypy/issues/1153
+    import json  # type: ignore
 import logging
 import os
 import re
 from collections import defaultdict
 from glob import glob
+from pathlib import Path
 from typing import Dict, List, Optional, Union
 
 import numpy as np
-import xarray as xr
 
 from .. import utils
 from ..rcparams import rcParams
-from .base import CoordSpec, DimSpec, dict_to_dataset, generate_dims_coords, requires
+from .base import CoordSpec, DimSpec, dict_to_dataset, infer_stan_dtypes, requires
 from .inference_data import InferenceData
 
 _log = logging.getLogger(__name__)
@@ -23,9 +31,7 @@ def check_glob(path, group, disable_glob):
         path_glob = glob(path)
         if path_glob:
             path = sorted(path_glob)
-            msg = "\n".join(
-                "{}: {}".format(i, os.path.normpath(fpath)) for i, fpath in enumerate(path, 1)
-            )
+            msg = "\n".join(f"{i}: {os.path.normpath(fpath)}" for i, fpath in enumerate(path, 1))
             len_p = len(path)
             _log.info("glob found %d files for '%s':\n%s", len_p, group, msg)
     return path
@@ -51,6 +57,7 @@ class CmdStanConverter:
         predictions_constant_data=None,
         predictions_constant_data_var=None,
         log_likelihood=None,
+        index_origin=None,
         coords=None,
         dims=None,
         disable_glob=False,
@@ -80,15 +87,35 @@ class CmdStanConverter:
         self.attrs_prior = None
 
         self.save_warmup = rcParams["data.save_warmup"] if save_warmup is None else save_warmup
+        self.index_origin = index_origin
 
         if dtypes is None:
-            self.dtypes = {}
-        else:
-            self.dtypes = dtypes
+            dtypes = {}
+        elif isinstance(dtypes, str):
+            dtypes_path = Path(dtypes)
+            if dtypes_path.exists():
+                with dtypes_path.open("r", encoding="UTF-8") as f_obj:
+                    model_code = f_obj.read()
+            else:
+                model_code = dtypes
+
+            dtypes = infer_stan_dtypes(model_code)
+
+        self.dtypes = dtypes
+
         # populate posterior and sample_stats
         self._parse_posterior()
         self._parse_prior()
 
+        if (
+            self.log_likelihood is None
+            and self.posterior_ is not None
+            and any(name.split(".")[0] == "log_lik" for name in self.posterior_columns)
+        ):
+            self.log_likelihood = ["log_lik"]
+        elif isinstance(self.log_likelihood, bool):
+            self.log_likelihood = None
+
     @requires("posterior_")
     def _parse_posterior(self):
         """Read csv paths to list of ndarrays."""
@@ -190,6 +217,12 @@ class CmdStanConverter:
             log_likelihood = []
         elif isinstance(log_likelihood, str):
             log_likelihood = [col for col in columns if log_likelihood == col.split(".")[0]]
+        elif isinstance(log_likelihood, dict):
+            log_likelihood = [
+                col
+                for col in columns
+                if any(item == col.split(".")[0] for item in log_likelihood.values())
+            ]
         else:
             log_likelihood = [
                 col for col in columns if any(item == col.split(".")[0] for item in log_likelihood)
@@ -200,8 +233,20 @@ class CmdStanConverter:
         data = _unpack_ndarrays(self.posterior[0], valid_cols, self.dtypes)
         data_warmup = _unpack_ndarrays(self.posterior[1], valid_cols, self.dtypes)
         return (
-            dict_to_dataset(data, coords=self.coords, dims=self.dims, attrs=self.attrs),
-            dict_to_dataset(data_warmup, coords=self.coords, dims=self.dims, attrs=self.attrs),
+            dict_to_dataset(
+                data,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=self.attrs,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=self.attrs,
+                index_origin=self.index_origin,
+            ),
         )
 
     @requires("posterior")
@@ -231,12 +276,14 @@ class CmdStanConverter:
                 coords=self.coords,
                 dims=self.dims,
                 attrs={item: key for key, item in rename_dict.items()},
+                index_origin=self.index_origin,
             ),
             dict_to_dataset(
                 data_warmup,
                 coords=self.coords,
                 dims=self.dims,
                 attrs={item: key for key, item in rename_dict.items()},
+                index_origin=self.index_origin,
             ),
         )
 
@@ -284,8 +331,20 @@ class CmdStanConverter:
 
             attrs = None
         return (
-            dict_to_dataset(data, coords=self.coords, dims=self.dims, attrs=attrs),
-            dict_to_dataset(data_warmup, coords=self.coords, dims=self.dims, attrs=attrs),
+            dict_to_dataset(
+                data,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=attrs,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=attrs,
+                index_origin=self.index_origin,
+            ),
         )
 
     @requires("posterior")
@@ -330,8 +389,20 @@ class CmdStanConverter:
 
             attrs = None
         return (
-            dict_to_dataset(data, coords=self.coords, dims=self.dims, attrs=attrs),
-            dict_to_dataset(data_warmup, coords=self.coords, dims=self.dims, attrs=attrs),
+            dict_to_dataset(
+                data,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=attrs,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=attrs,
+                index_origin=self.index_origin,
+            ),
         )
 
     @requires("posterior")
@@ -365,22 +436,42 @@ class CmdStanConverter:
             data = _unpack_ndarrays(chain_data, columns, self.dtypes)
             data_warmup = _unpack_ndarrays(chain_data_warmup, columns, self.dtypes)
         else:
-            if isinstance(log_likelihood, str):
-                log_likelihood = [log_likelihood]
-            columns = {
-                col: idx
-                for col, idx in self.posterior_columns.items()
-                if any(item == col.split(".")[0] for item in log_likelihood)
-            }
+            if isinstance(log_likelihood, dict):
+                log_lik_to_obs_name = {v: k for k, v in log_likelihood.items()}
+                columns = {
+                    col.replace(col_name, log_lik_to_obs_name[col_name]): idx
+                    for col, col_name, idx in (
+                        (col, col.split(".")[0], idx) for col, idx in self.posterior_columns.items()
+                    )
+                    if any(item == col_name for item in log_likelihood.values())
+                }
+            else:
+                if isinstance(log_likelihood, str):
+                    log_likelihood = [log_likelihood]
+                columns = {
+                    col: idx
+                    for col, idx in self.posterior_columns.items()
+                    if any(item == col.split(".")[0] for item in log_likelihood)
+                }
             data = _unpack_ndarrays(self.posterior[0], columns, self.dtypes)
             data_warmup = _unpack_ndarrays(self.posterior[1], columns, self.dtypes)
             attrs = None
         return (
             dict_to_dataset(
-                data, coords=self.coords, dims=self.dims, attrs=attrs, skip_event_dims=True
+                data,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=attrs,
+                index_origin=self.index_origin,
+                skip_event_dims=True,
             ),
             dict_to_dataset(
-                data_warmup, coords=self.coords, dims=self.dims, attrs=attrs, skip_event_dims=True
+                data_warmup,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=attrs,
+                index_origin=self.index_origin,
+                skip_event_dims=True,
             ),
         )
 
@@ -410,9 +501,19 @@ class CmdStanConverter:
         data = _unpack_ndarrays(self.prior[0], valid_cols, self.dtypes)
         data_warmup = _unpack_ndarrays(self.prior[1], valid_cols, self.dtypes)
         return (
-            dict_to_dataset(data, coords=self.coords, dims=self.dims, attrs=self.attrs_prior),
             dict_to_dataset(
-                data_warmup, coords=self.coords, dims=self.dims, attrs=self.attrs_prior
+                data,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=self.attrs_prior,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=self.attrs_prior,
+                index_origin=self.index_origin,
             ),
         )
 
@@ -443,12 +544,14 @@ class CmdStanConverter:
                 coords=self.coords,
                 dims=self.dims,
                 attrs={item: key for key, item in rename_dict.items()},
+                index_origin=self.index_origin,
             ),
             dict_to_dataset(
                 data_warmup,
                 coords=self.coords,
                 dims=self.dims,
                 attrs={item: key for key, item in rename_dict.items()},
+                index_origin=self.index_origin,
             ),
         )
 
@@ -491,8 +594,20 @@ class CmdStanConverter:
             data_warmup = _unpack_ndarrays(self.prior[1], columns, self.dtypes)
             attrs = None
         return (
-            dict_to_dataset(data, coords=self.coords, dims=self.dims, attrs=attrs),
-            dict_to_dataset(data_warmup, coords=self.coords, dims=self.dims, attrs=attrs),
+            dict_to_dataset(
+                data,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=attrs,
+                index_origin=self.index_origin,
+            ),
+            dict_to_dataset(
+                data_warmup,
+                coords=self.coords,
+                dims=self.dims,
+                attrs=attrs,
+                index_origin=self.index_origin,
+            ),
         )
 
     @requires("observed_data")
@@ -506,13 +621,14 @@ class CmdStanConverter:
         for key, vals in observed_data_raw.items():
             if variables is not None and key not in variables:
                 continue
-            vals = utils.one_de(vals)
-            val_dims = self.dims.get(key)
-            val_dims, coords = generate_dims_coords(
-                vals.shape, key, dims=val_dims, coords=self.coords
-            )
-            observed_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
-        return xr.Dataset(data_vars=observed_data)
+            observed_data[key] = utils.one_de(vals)
+        return dict_to_dataset(
+            observed_data,
+            coords=self.coords,
+            dims=self.dims,
+            default_dims=[],
+            index_origin=self.index_origin,
+        )
 
     @requires("constant_data")
     def constant_data_to_xarray(self):
@@ -525,13 +641,14 @@ class CmdStanConverter:
         for key, vals in constant_data_raw.items():
             if variables is not None and key not in variables:
                 continue
-            vals = utils.one_de(vals)
-            val_dims = self.dims.get(key)
-            val_dims, coords = generate_dims_coords(
-                vals.shape, key, dims=val_dims, coords=self.coords
-            )
-            constant_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
-        return xr.Dataset(data_vars=constant_data)
+            constant_data[key] = utils.one_de(vals)
+        return dict_to_dataset(
+            constant_data,
+            coords=self.coords,
+            dims=self.dims,
+            default_dims=[],
+            index_origin=self.index_origin,
+        )
 
     @requires("predictions_constant_data")
     def predictions_constant_data_to_xarray(self):
@@ -545,12 +662,14 @@ class CmdStanConverter:
             if variables is not None and key not in variables:
                 continue
             vals = utils.one_de(vals)
-            val_dims = self.dims.get(key)
-            val_dims, coords = generate_dims_coords(
-                vals.shape, key, dims=val_dims, coords=self.coords
-            )
-            predictions_constant_data[key] = xr.DataArray(vals, dims=val_dims, coords=coords)
-        return xr.Dataset(data_vars=predictions_constant_data)
+            predictions_constant_data[key] = utils.one_de(vals)
+        return dict_to_dataset(
+            predictions_constant_data,
+            coords=self.coords,
+            dims=self.dims,
+            default_dims=[],
+            index_origin=self.index_origin,
+        )
 
     def to_inference_data(self):
         """Convert all available data to an InferenceData object.
@@ -740,7 +859,7 @@ def _process_data_var(string):
 
 
 def _read_data(path):
-    """Read Rdump output to dictionary.
+    """Read Rdump or JSON output to dictionary.
 
     Parameters
     ----------
@@ -749,10 +868,12 @@ def _read_data(path):
     Returns
     -------
     Dict
-        key, values pairs from Rdump formatted data.
+        key, values pairs from Rdump/JSON formatted data.
     """
     data = {}
-    with open(path, "r") as f_obj:
+    with open(path, "r", encoding="utf8") as f_obj:
+        if path.lower().endswith(".json"):
+            return json.load(f_obj)
         var = ""
         for line in f_obj:
             if "<-" in line:
@@ -794,7 +915,7 @@ def _unpack_ndarrays(arrays, columns, dtypes=None):
         for key, cols_locs in col_groups.items():
             ndim = np.array([loc for _, loc in cols_locs]).max(0) + 1
             dtype = dtypes.get(key, np.float64)
-            sample[key] = utils.full((chains, draws, *ndim), 0, dtype=dtype)
+            sample[key] = np.zeros((chains, draws, *ndim), dtype=dtype)
             for col, loc in cols_locs:
                 for chain_id, arr in enumerate(arrays):
                     draw = arr[:, col]
@@ -821,6 +942,7 @@ def from_cmdstan(
     predictions_constant_data: Optional[str] = None,
     predictions_constant_data_var: Optional[Union[str, List[str]]] = None,
     log_likelihood: Optional[Union[str, List[str]]] = None,
+    index_origin: Optional[int] = None,
     coords: Optional[CoordSpec] = None,
     dims: Optional[DimSpec] = None,
     disable_glob: Optional[bool] = False,
@@ -845,27 +967,36 @@ def from_cmdstan(
     prior_predictive : str or list of str, optional
         Prior predictive samples for the fit. If endswith ".csv" assumes file.
     observed_data : str, optional
-        Observed data used in the sampling. Path to data file in Rdump format.
+        Observed data used in the sampling. Path to data file in Rdump or JSON format.
     observed_data_var : str or list of str, optional
         Variable(s) used for slicing observed_data. If not defined, all
         data variables are imported.
     constant_data : str, optional
-        Constant data used in the sampling. Path to data file in Rdump format.
+        Constant data used in the sampling. Path to data file in Rdump or JSON format.
     constant_data_var : str or list of str, optional
         Variable(s) used for slicing constant_data. If not defined, all
         data variables are imported.
     predictions_constant_data : str, optional
         Constant data for predictions used in the sampling.
-        Path to data file in Rdump format.
+        Path to data file in Rdump or JSON format.
     predictions_constant_data_var : str or list of str, optional
         Variable(s) used for slicing predictions_constant_data.
         If not defined, all data variables are imported.
-    log_likelihood : str or list of str, optional
-        Pointwise log_likelihood for the data.
+    log_likelihood : dict of {str: str}, list of str or str, optional
+        Pointwise log_likelihood for the data. log_likelihood is extracted from the
+        posterior. It is recommended to use this argument as a dictionary whose keys
+        are observed variable names and its values are the variables storing log
+        likelihood arrays in the Stan code. In other cases, a dictionary with keys
+        equal to its values is used. By default, if a variable ``log_lik`` is
+        present in the Stan model, it will be retrieved as pointwise log
+        likelihood values. Use ``False`` to avoid this behaviour.
+    index_origin : int, optional
+        Starting value of integer coordinate values. Defaults to the value in rcParam
+        ``data.index_origin``.
     coords : dict of {str: array_like}, optional
         A dictionary containing the values that are used as index. The key
         is the name of the dimension, the values are the index values.
-    dims : dict of {str: list of str, optional
+    dims : dict of {str: list of str}, optional
         A mapping from variables to a list of coordinate names for the variable.
     disable_glob : bool
         Don't use glob for string input. This means that all string input is
@@ -873,8 +1004,9 @@ def from_cmdstan(
     save_warmup : bool
         Save warmup iterations into InferenceData object, if found in the input files.
         If not defined, use default defined by the rcParams.
-    dtypes : dict
+    dtypes : dict or str
         A dictionary containing dtype information (int, float) for parameters.
+        If input is a string, it is assumed to be a model code or path to model code file.
 
     Returns
     -------
@@ -893,6 +1025,7 @@ def from_cmdstan(
         predictions_constant_data=predictions_constant_data,
         predictions_constant_data_var=predictions_constant_data_var,
         log_likelihood=log_likelihood,
+        index_origin=index_origin,
         coords=coords,
         dims=dims,
         disable_glob=disable_glob,
