@@ -72,7 +72,10 @@ class CmdStanPyConverter:
 
         self.dtypes = dtypes
 
-        if hasattr(self.posterior, "stan_vars_cols"):
+        if hasattr(self.posterior, "metadata"):
+            if self.log_likelihood is True and "log_lik" in self.posterior.metadata.stan_vars_cols:
+                self.log_likelihood = ["log_lik"]
+        elif hasattr(self.posterior, "stan_vars_cols"):
             if self.log_likelihood is True and "log_lik" in self.posterior.stan_vars_cols:
                 self.log_likelihood = ["log_lik"]
         else:
@@ -91,10 +94,13 @@ class CmdStanPyConverter:
     @requires("posterior")
     def posterior_to_xarray(self):
         """Extract posterior samples from output csv."""
-        if not hasattr(self.posterior, "stan_vars_cols"):
+        if not (hasattr(self.posterior, "metadata") or hasattr(self.posterior, "stan_vars_cols")):
             return self.posterior_to_xarray_pre_v_0_9_68()
 
-        items = list(self.posterior.stan_vars_cols.keys())
+        if hasattr(self.posterior, "metadata"):
+            items = list(self.posterior.metadata.stan_vars_cols.keys())
+        else:
+            items = list(self.posterior.stan_vars_cols.keys())
         if self.posterior_predictive is not None:
             try:
                 items = _filter(items, self.posterior_predictive)
@@ -113,7 +119,10 @@ class CmdStanPyConverter:
 
         valid_cols = []
         for item in items:
-            valid_cols.extend(self.posterior.stan_vars_cols[item])
+            if hasattr(self.posterior, "metadata"):
+                valid_cols.extend(self.posterior.metadata.stan_vars_cols[item])
+            else:
+                valid_cols.extend(self.posterior.stan_vars_cols[item])
 
         data, data_warmup = _unpack_fit(
             self.posterior,
@@ -155,7 +164,7 @@ class CmdStanPyConverter:
 
     def stats_to_xarray(self, fit):
         """Extract sample_stats from fit."""
-        if not hasattr(fit, "sampler_vars_cols"):
+        if not (hasattr(fit, "metadata") or hasattr(fit, "sampler_vars_cols")):
             return self.sample_stats_to_xarray_pre_v_0_9_68(fit)
 
         dtypes = {
@@ -164,7 +173,10 @@ class CmdStanPyConverter:
             "treedepth__": np.int64,
             **self.dtypes,
         }
-        items = list(fit.sampler_vars_cols.keys())
+        if hasattr(fit, "metadata"):
+            items = list(fit.metadata._method_vars_cols.keys())  # pylint: disable=protected-access
+        else:
+            items = list(fit.sampler_vars_cols.keys())
         rename_dict = {
             "divergent": "diverging",
             "n_leapfrog": "n_steps",
@@ -218,7 +230,7 @@ class CmdStanPyConverter:
         """Convert predictive samples to xarray."""
         predictive = _as_set(names)
 
-        if hasattr(fit, "stan_vars_cols"):
+        if hasattr(fit, "metadata") or hasattr(fit, "stan_vars_cols"):
             data, data_warmup = _unpack_fit(
                 fit,
                 predictive,
@@ -258,7 +270,7 @@ class CmdStanPyConverter:
         """Convert out of sample predictions samples to xarray."""
         predictions = _as_set(self.predictions)
 
-        if hasattr(self.posterior, "stan_vars_cols"):
+        if hasattr(self.posterior, "metadata") or hasattr(self.posterior, "stan_vars_cols"):
             data, data_warmup = _unpack_fit(
                 self.posterior,
                 predictions,
@@ -299,7 +311,7 @@ class CmdStanPyConverter:
         """Convert elementwise log likelihood samples to xarray."""
         log_likelihood = _as_set(self.log_likelihood)
 
-        if hasattr(self.posterior, "stan_vars_cols"):
+        if hasattr(self.posterior, "metadata") or hasattr(self.posterior, "stan_vars_cols"):
             data, data_warmup = _unpack_fit(
                 self.posterior,
                 log_likelihood,
@@ -345,8 +357,11 @@ class CmdStanPyConverter:
     @requires("prior")
     def prior_to_xarray(self):
         """Convert prior samples to xarray."""
-        if hasattr(self.prior, "stan_vars_cols"):
-            items = list(self.prior.stan_vars_cols.keys())
+        if hasattr(self.posterior, "metadata") or hasattr(self.prior, "stan_vars_cols"):
+            if hasattr(self.posterior, "metadata"):
+                items = list(self.prior.metadata.stan_vars_cols.keys())
+            else:
+                items = list(self.prior.stan_vars_cols.keys())
             if self.prior_predictive is not None:
                 try:
                     items = _filter(items, self.prior_predictive)
@@ -624,9 +639,15 @@ def _unpack_fit(fit, items, save_warmup, dtypes):
     sample = {}
     sample_warmup = {}
 
+    stan_vars_cols = fit.metadata.stan_vars_cols if hasattr(fit, "metadata") else fit.stan_vars_cols
+    sampler_vars_cols = (
+        fit.metadata._method_vars_cols  # pylint: disable=protected-access
+        if hasattr(fit, "metadata")
+        else fit.sampler_vars_cols
+    )
     for item in items:
-        if item in fit.stan_vars_cols:
-            col_idxs = fit.stan_vars_cols[item]
+        if item in stan_vars_cols:
+            col_idxs = stan_vars_cols[item]
             if len(col_idxs) == 1:
                 raw_draws = draws[..., col_idxs[0]]
             else:
@@ -634,8 +655,8 @@ def _unpack_fit(fit, items, save_warmup, dtypes):
                 raw_draws = np.swapaxes(
                     raw_draws.reshape((-1, nchains, *raw_draws.shape[1:]), order="F"), 0, 1
                 )
-        elif item in fit.sampler_vars_cols:
-            col_idxs = fit.sampler_vars_cols[item]
+        elif item in sampler_vars_cols:
+            col_idxs = sampler_vars_cols[item]
             raw_draws = draws[..., col_idxs[0]]
         else:
             raise ValueError(f"fit data, unknown variable: {item}")
