@@ -5,14 +5,16 @@ from .compareplot import plot_compare
 from .densityplot import plot_density
 from .distcomparisonplot import plot_dist_comparison
 from .distplot import plot_dist
+from .dotplot import plot_dot
+from .ecdfplot import plot_ecdf
 from .elpdplot import plot_elpd
 from .energyplot import plot_energy
 from .essplot import plot_ess
 from .forestplot import plot_forest
 from .hdiplot import plot_hdi
-from .jointplot import plot_joint
 from .kdeplot import plot_kde
 from .khatplot import plot_khat
+from .lmplot import plot_lm
 from .loopitplot import plot_loo_pit
 from .mcseplot import plot_mcse
 from .pairplot import plot_pair
@@ -22,6 +24,7 @@ from .ppcplot import plot_ppc
 from .rankplot import plot_rank
 from .separationplot import plot_separation
 from .traceplot import plot_trace
+from .tsplot import plot_ts
 from .violinplot import plot_violin
 
 __all__ = [
@@ -30,14 +33,16 @@ __all__ = [
     "plot_compare",
     "plot_density",
     "plot_dist",
+    "plot_dot",
+    "plot_ecdf",
     "plot_elpd",
     "plot_energy",
     "plot_ess",
     "plot_forest",
     "plot_hdi",
-    "plot_joint",
     "plot_kde",
     "plot_khat",
+    "plot_lm",
     "plot_loo_pit",
     "plot_mcse",
     "plot_pair",
@@ -47,6 +52,7 @@ __all__ = [
     "plot_dist_comparison",
     "plot_rank",
     "plot_trace",
+    "plot_ts",
     "plot_violin",
     "plot_separation",
 ]
