@@ -12,8 +12,8 @@ from ...plot_utils import (
     _scale_fig_size,
     calculate_point_estimate,
     format_sig_figs,
-    make_label,
     round_num,
+    vectorized_to_hex,
 )
 from .. import show_layout
 from . import backend_kwarg_defaults, create_axes_grid
@@ -38,6 +38,9 @@ def plot_posterior(
     textsize,
     ref_val,
     rope,
+    ref_val_color,
+    rope_color,
+    labeller,
     kwargs,
     backend_kwargs,
     show,
@@ -66,7 +69,7 @@ def plot_posterior(
     else:
         ax = np.atleast_2d(ax)
     idx = 0
-    for (var_name, selection, x), ax_ in zip(
+    for (var_name, selection, isel, x), ax_ in zip(
         plotters, (item for item in ax.flatten() if item is not None)
     ):
         _plot_posterior_op(
@@ -87,12 +90,14 @@ def plot_posterior(
             linewidth=linewidth,
             ref_val=ref_val,
             rope=rope,
+            ref_val_color=ref_val_color,
+            rope_color=rope_color,
             ax_labelsize=ax_labelsize,
             **kwargs,
         )
         idx += 1
         _title = Title()
-        _title.text = make_label(var_name, selection)
+        _title.text = labeller.make_label_vert(var_name, selection, isel)
         ax_.title = _title
 
     show_layout(ax, show)
@@ -117,6 +122,8 @@ def _plot_posterior_op(
     skipna,
     ref_val,
     rope,
+    ref_val_color,
+    rope_color,
     ax_labelsize,
     round_to: Optional[int] = None,
     **kwargs,
@@ -155,9 +162,20 @@ def _plot_posterior_op(
             val,
             format_as_percent(greater_than_ref_probability, 1),
         )
-        ax.line([val, val], [0, 0.8 * max_data], line_color="blue", line_alpha=0.65)
+        ax.line(
+            [val, val],
+            [0, 0.8 * max_data],
+            line_color=vectorized_to_hex(ref_val_color),
+            line_alpha=0.65,
+        )
 
-        ax.text(x=[values.mean()], y=[max_data * 0.6], text=[ref_in_posterior], text_align="center")
+        ax.text(
+            x=[values.mean()],
+            y=[max_data * 0.6],
+            text=[ref_in_posterior],
+            text_color=vectorized_to_hex(ref_val_color),
+            text_align="center",
+        )
 
     def display_rope(max_data):
         if rope is None:
@@ -185,15 +203,28 @@ def _plot_posterior_op(
             vals,
             (max_data * 0.02, max_data * 0.02),
             line_width=linewidth * 5,
-            line_color="red",
+            line_color=vectorized_to_hex(rope_color),
             line_alpha=0.7,
         )
-
+        probability_within_rope = ((values > vals[0]) & (values <= vals[1])).mean()
         text_props = dict(
-            text_font_size="{}pt".format(ax_labelsize), text_color="black", text_align="center"
+            text_color=vectorized_to_hex(rope_color),
+            text_align="center",
+        )
+        ax.text(
+            x=values.mean(),
+            y=[max_data * 0.45],
+            text=[f"{format_as_percent(probability_within_rope, 1)} in ROPE"],
+            **text_props,
         )
 
-        ax.text(x=vals, y=[max_data * 0.2, max_data * 0.2], text=rope_text, **text_props)
+        ax.text(
+            x=vals,
+            y=[max_data * 0.2, max_data * 0.2],
+            text_font_size=f"{ax_labelsize}pt",
+            text=rope_text,
+            **text_props,
+        )
 
     def display_point_estimate(max_data):
         if not point_estimate:
@@ -245,7 +276,7 @@ def _plot_posterior_op(
         plot_kde(
             values,
             bw=bw,
-            circular=circular,
+            is_circular=circular,
             fill_kwargs={"fill_alpha": kwargs.pop("fill_alpha", 0)},
             plot_kwargs=kwargs,
             ax=ax,
@@ -255,18 +286,30 @@ def _plot_posterior_op(
             show=False,
         )
         _, hist, edges = histogram(values, bins="auto")
-    else:
+    elif values.dtype.kind == "i" or (values.dtype.kind == "f" and kind == "hist"):
         if bins is None:
-            if values.dtype.kind == "i":
-                bins = get_bins(values)
-            else:
-                bins = "auto"
+            bins = get_bins(values)
         kwargs.setdefault("align", "left")
         kwargs.setdefault("color", "blue")
         _, hist, edges = histogram(values, bins=bins)
         ax.quad(
             top=hist, bottom=0, left=edges[:-1], right=edges[1:], fill_alpha=0.35, line_alpha=0.35
         )
+    elif values.dtype.kind == "b":
+        if bins is None:
+            bins = "auto"
+        kwargs.setdefault("color", "blue")
+
+        hist = np.array([(~values).sum(), values.sum()])
+        edges = np.array([-0.5, 0.5, 1.5])
+        ax.quad(
+            top=hist, bottom=0, left=edges[:-1], right=edges[1:], fill_alpha=0.35, line_alpha=0.35
+        )
+        hdi_prob = "hide"
+        ax.xaxis.ticker = [0, 1]
+        ax.xaxis.major_label_overrides = {0: "False", 1: "True"}
+    else:
+        raise TypeError("Values must be float, integer or boolean")
 
     format_axes()
     max_data = hist.max()
