@@ -1,5 +1,6 @@
 """Bokeh Posterior predictive plot."""
 import numpy as np
+from bokeh.models.annotations import Legend
 
 from ....stats.density_utils import get_bins, histogram, kde
 from ...kdeplot import plot_kde
@@ -22,13 +23,14 @@ def plot_ppc(
     pp_sample_ix,
     kind,
     alpha,
-    color,
+    colors,
     textsize,
     mean,
     observed,
     jitter,
     total_pp_samples,
     legend,  # pylint: disable=unused-argument
+    labeller,
     group,  # pylint: disable=unused-argument
     animation_kwargs,  # pylint: disable=unused-argument
     num_pp_samples,
@@ -46,7 +48,7 @@ def plot_ppc(
         **backend_kwargs,
     }
 
-    color = vectorized_to_hex(color)
+    colors = vectorized_to_hex(colors)
 
     (figsize, *_, linewidth, markersize) = _scale_fig_size(figsize, textsize, rows, cols)
     if ax is None:
@@ -82,9 +84,15 @@ def plot_ppc(
         raise ValueError("jitter must be >=0.")
 
     for i, ax_i in enumerate((item for item in axes.flatten() if item is not None)):
-        var_name, _, obs_vals = obs_plotters[i]
-        pp_var_name, _, pp_vals = pp_plotters[i]
+        var_name, sel, isel, obs_vals = obs_plotters[i]
+        pp_var_name, _, _, pp_vals = pp_plotters[i]
         dtype = predictive_dataset[pp_var_name].dtype.kind
+        legend_it = []
+
+        if dtype not in ["i", "f"]:
+            raise ValueError(
+                f"The data type of the predictive data must be one of 'i' or 'f', but is '{dtype}'"
+            )
 
         # flatten non-specified dimensions
         obs_vals = obs_vals.flatten()
@@ -92,7 +100,11 @@ def plot_ppc(
         pp_sampled_vals = pp_vals[pp_sample_ix]
 
         if kind == "kde":
-            plot_kwargs = {"line_color": color, "line_alpha": alpha, "line_width": 0.5 * linewidth}
+            plot_kwargs = {
+                "line_color": colors[0],
+                "line_alpha": alpha,
+                "line_width": 0.5 * linewidth,
+            }
 
             pp_densities = []
             pp_xs = []
@@ -110,35 +122,44 @@ def plot_ppc(
                     pp_xs.append(bin_edges)
 
             if dtype == "f":
-                ax_i.multi_line(pp_xs, pp_densities, **plot_kwargs)
+                multi_line = ax_i.multi_line(pp_xs, pp_densities, **plot_kwargs)
+                legend_it.append((f"{group.capitalize()} predictive", [multi_line]))
             else:
+                all_steps = []
                 for x_s, y_s in zip(pp_xs, pp_densities):
-                    ax_i.step(x_s, y_s, **plot_kwargs)
+                    step = ax_i.step(x_s, y_s, **plot_kwargs)
+                    all_steps.append(step)
+                legend_it.append((f"{group.capitalize()} predictive", all_steps))
 
             if observed:
+                label = "Observed"
                 if dtype == "f":
-                    plot_kde(
+                    _, glyph = plot_kde(
                         obs_vals,
-                        plot_kwargs={"line_color": "black", "line_width": linewidth},
+                        plot_kwargs={"line_color": colors[1], "line_width": linewidth},
                         fill_kwargs={"alpha": 0},
                         ax=ax_i,
                         backend="bokeh",
                         backend_kwargs={},
                         show=False,
+                        return_glyph=True,
                     )
+                    legend_it.append((label, glyph))
                 else:
                     bins = get_bins(obs_vals)
                     _, hist, bin_edges = histogram(obs_vals, bins=bins)
                     hist = np.concatenate((hist[:1], hist))
-                    ax_i.step(
+                    step = ax_i.step(
                         bin_edges,
                         hist,
-                        line_color="black",
+                        line_color=colors[1],
                         line_width=linewidth,
                         mode="center",
                     )
+                    legend_it.append((label, [step]))
 
             if mean:
+                label = f"{group.capitalize()} predictive mean"
                 if dtype == "f":
                     rep = len(pp_densities)
                     len_density = len(pp_densities[0])
@@ -149,75 +170,84 @@ def plot_ppc(
                     new_x -= (new_x[1] - new_x[0]) / 2
                     for irep in range(rep):
                         new_d[irep][bins[irep]] = pp_densities[irep]
-                    ax_i.line(
+                    line = ax_i.line(
                         new_x,
                         new_d.mean(0),
-                        color=color,
+                        color=colors[2],
                         line_dash="dashed",
                         line_width=linewidth,
                     )
+                    legend_it.append((label, [line]))
                 else:
                     vals = pp_vals.flatten()
                     bins = get_bins(vals)
                     _, hist, bin_edges = histogram(vals, bins=bins)
                     hist = np.concatenate((hist[:1], hist))
-                    ax_i.step(
+                    step = ax_i.step(
                         bin_edges,
                         hist,
-                        line_color=color,
+                        line_color=colors[2],
                         line_width=linewidth,
                         line_dash="dashed",
                         mode="center",
                     )
+                    legend_it.append((label, [step]))
             ax_i.yaxis.major_tick_line_color = None
             ax_i.yaxis.minor_tick_line_color = None
             ax_i.yaxis.major_label_text_font_size = "0pt"
 
         elif kind == "cumulative":
             if observed:
+                label = "Observed"
                 if dtype == "f":
                     glyph = ax_i.line(
                         *_empirical_cdf(obs_vals),
-                        line_color="black",
+                        line_color=colors[1],
                         line_width=linewidth,
                     )
                     glyph.level = "overlay"
+                    legend_it.append((label, [glyph]))
 
                 else:
-                    ax_i.step(
+                    step = ax_i.step(
                         *_empirical_cdf(obs_vals),
-                        line_color="black",
+                        line_color=colors[1],
                         line_width=linewidth,
                         mode="center",
                     )
+                    legend_it.append((label, [step]))
             pp_densities = np.empty((2 * len(pp_sampled_vals), pp_sampled_vals[0].size))
             for idx, vals in enumerate(pp_sampled_vals):
                 vals = np.array([vals]).flatten()
                 pp_x, pp_density = _empirical_cdf(vals)
                 pp_densities[2 * idx] = pp_x
                 pp_densities[2 * idx + 1] = pp_density
-            ax_i.multi_line(
+            multi_line = ax_i.multi_line(
                 list(pp_densities[::2]),
                 list(pp_densities[1::2]),
                 line_alpha=alpha,
-                line_color=color,
+                line_color=colors[0],
                 line_width=linewidth,
             )
+            legend_it.append((f"{group.capitalize()} predictive", [multi_line]))
             if mean:
-                ax_i.line(
+                label = f"{group.capitalize()} predictive mean"
+                line = ax_i.line(
                     *_empirical_cdf(pp_vals.flatten()),
-                    color=color,
+                    color=colors[2],
                     line_dash="dashed",
                     line_width=linewidth,
                 )
+                legend_it.append((label, [line]))
 
         elif kind == "scatter":
             if mean:
+                label = f"{group.capitalize()} predictive mean"
                 if dtype == "f":
-                    plot_kde(
+                    _, glyph = plot_kde(
                         pp_vals.flatten(),
                         plot_kwargs={
-                            "line_color": color,
+                            "line_color": colors[2],
                             "line_dash": "dashed",
                             "line_width": linewidth,
                         },
@@ -225,20 +255,23 @@ def plot_ppc(
                         backend="bokeh",
                         backend_kwargs={},
                         show=False,
+                        return_glyph=True,
                     )
+                    legend_it.append((label, glyph))
                 else:
                     vals = pp_vals.flatten()
                     bins = get_bins(vals)
                     _, hist, bin_edges = histogram(vals, bins=bins)
                     hist = np.concatenate((hist[:1], hist))
-                    ax_i.step(
+                    step = ax_i.step(
                         bin_edges,
                         hist,
-                        color=color,
+                        color=colors[2],
                         line_width=linewidth,
                         line_dash="dashed",
                         mode="center",
                     )
+                    legend_it.append((label, [step]))
 
             jitter_scale = 0.1
             y_rows = np.linspace(0, 0.1, num_pp_samples + 1)
@@ -246,6 +279,7 @@ def plot_ppc(
             scale_high = jitter_scale * jitter
 
             if observed:
+                label = "Observed"
                 obs_yvals = np.zeros_like(obs_vals, dtype=np.float64)
                 if jitter:
                     obs_yvals += np.random.uniform(
@@ -254,28 +288,46 @@ def plot_ppc(
                 glyph = ax_i.circle(
                     obs_vals,
                     obs_yvals,
-                    fill_color="black",
+                    line_color=colors[1],
+                    fill_color=colors[1],
                     size=markersize,
                     line_alpha=alpha,
                 )
                 glyph.level = "overlay"
+                legend_it.append((label, [glyph]))
 
+            all_scatter = []
             for vals, y in zip(pp_sampled_vals, y_rows[1:]):
                 vals = np.ravel(vals)
                 yvals = np.full_like(vals, y, dtype=np.float64)
                 if jitter:
                     yvals += np.random.uniform(low=scale_low, high=scale_high, size=len(vals))
-                ax_i.scatter(vals, yvals, fill_color=color, size=markersize, fill_alpha=alpha)
+                scatter = ax_i.scatter(
+                    vals,
+                    yvals,
+                    line_color=colors[0],
+                    fill_color=colors[0],
+                    size=markersize,
+                    fill_alpha=alpha,
+                )
+                all_scatter.append(scatter)
 
+            legend_it.append((f"{group.capitalize()} predictive", all_scatter))
             ax_i.yaxis.major_tick_line_color = None
             ax_i.yaxis.minor_tick_line_color = None
             ax_i.yaxis.major_label_text_font_size = "0pt"
 
-        if var_name != pp_var_name:
-            xlabel = "{} / {}".format(var_name, pp_var_name)
-        else:
-            xlabel = var_name
-        ax_i.xaxis.axis_label = xlabel
+        if legend:
+            legend = Legend(
+                items=legend_it,
+                location="top_left",
+                orientation="vertical",
+            )
+            ax_i.add_layout(legend)
+            if textsize is not None:
+                ax_i.legend.label_text_font_size = f"{textsize}pt"
+            ax_i.legend.click_policy = "hide"
+        ax_i.xaxis.axis_label = labeller.make_pp_label(var_name, pp_var_name, sel, isel)
 
     show_layout(axes, show)
 
