@@ -5,12 +5,14 @@ from itertools import tee
 import matplotlib.pyplot as plt
 import numpy as np
 from matplotlib.colors import to_rgba
+from matplotlib.lines import Line2D
 
 from ....stats import hdi
 from ....stats.density_utils import get_bins, histogram, kde
 from ....stats.diagnostics import _ess, _rhat
+from ....sel_utils import xarray_var_iter
 from ....utils import conditional_jit
-from ...plot_utils import _scale_fig_size, make_label, xarray_var_iter
+from ...plot_utils import _scale_fig_size
 from . import backend_kwarg_defaults, backend_show
 
 
@@ -27,6 +29,7 @@ def plot_forest(
     var_names,
     model_names,
     combined,
+    combine_dims,
     colors,
     figsize,
     width_ratios,
@@ -43,6 +46,8 @@ def plot_forest(
     ridgeplot_truncate,
     ridgeplot_quantiles,
     textsize,
+    legend,
+    labeller,
     ess,
     r_hat,
     backend_kwargs,
@@ -51,7 +56,13 @@ def plot_forest(
 ):
     """Matplotlib forest plot."""
     plot_handler = PlotHandler(
-        datasets, var_names=var_names, model_names=model_names, combined=combined, colors=colors
+        datasets,
+        var_names=var_names,
+        model_names=model_names,
+        combined=combined,
+        combine_dims=combine_dims,
+        colors=colors,
+        labeller=labeller,
     )
 
     if figsize is None:
@@ -152,6 +163,8 @@ def plot_forest(
     if kind == "ridgeplot":  # space at the top
         y_max += ridgeplot_overlap
     axes[0].set_ylim(-all_plotters[0].group_offset, y_max)
+    if legend:
+        plot_handler.legend(ax=axes[0])
 
     if backend_show(show):
         plt.show()
@@ -164,14 +177,14 @@ class PlotHandler:
 
     # pylint: disable=inconsistent-return-statements
 
-    def __init__(self, datasets, var_names, model_names, combined, colors):
+    def __init__(self, datasets, var_names, model_names, combined, combine_dims, colors, labeller):
         self.data = datasets
 
         if model_names is None:
             if len(self.data) > 1:
-                model_names = ["Model {}".format(idx) for idx, _ in enumerate(self.data)]
+                model_names = [f"Model {idx}" for idx, _ in enumerate(self.data)]
             else:
-                model_names = [""]
+                model_names = [None]
         elif len(model_names) != len(self.data):
             raise ValueError("The number of model names does not match the number of models")
 
@@ -190,13 +203,16 @@ class PlotHandler:
             self.var_names = list(reversed(var_names))  # y-values are upside down
 
         self.combined = combined
+        self.combine_dims = combine_dims
 
         if colors == "cycle":
-            colors = ["C{}".format(idx) for idx, _ in enumerate(self.data)]
+            # TODO: Use matplotlib prop cycle instead
+            colors = [f"C{idx}" for idx, _ in enumerate(self.data)]
         elif isinstance(colors, str):
             colors = [colors for _ in self.data]
 
         self.colors = list(reversed(colors))  # y-values are upside down
+        self.labeller = labeller
 
         self.plotters = self.make_plotters()
 
@@ -210,7 +226,9 @@ class PlotHandler:
                 y,
                 model_names=self.model_names,
                 combined=self.combined,
+                combine_dims=self.combine_dims,
                 colors=self.colors,
+                labeller=self.labeller,
             )
             y = plotters[var_name].y_max()
         return plotters
@@ -224,12 +242,25 @@ class PlotHandler:
             labels, idxs = [], []
             for plotter in val:
                 sub_labels, sub_idxs, _, _ = plotter.labels_ticks_and_vals()
+                labels_to_idxs = defaultdict(list)
+                for label, idx in zip(sub_labels, sub_idxs):
+                    labels_to_idxs[label].append(idx)
+                sub_idxs = []
+                sub_labels = []
+                for label, all_idx in labels_to_idxs.items():
+                    sub_labels.append(label)
+                    sub_idxs.append(np.mean([j for j in all_idx]))
                 labels.append(sub_labels)
                 idxs.append(sub_idxs)
             return np.concatenate(labels), np.concatenate(idxs)
 
         return label_idxs()
 
+    def legend(self, ax):
+        """Add legend with colorcoded model info."""
+        handles = [Line2D([], [], color=c) for c in self.colors]
+        ax.legend(handles=handles, labels=self.model_names)
+
     def display_multiple_ropes(self, rope, ax, y, linewidth, var_name, selection):
         """Display ROPE when more than one interval is provided."""
         for sel in rope.get(var_name, []):
@@ -398,7 +429,7 @@ class PlotHandler:
                     color=color,
                 )
         ax.tick_params(labelsize=xt_labelsize)
-        ax.set_title("{:.1%} HDI".format(hdi_prob), fontsize=titlesize, wrap=True)
+        ax.set_title(f"{hdi_prob:.1%} HDI", fontsize=titlesize, wrap=True)
         if rope is None or isinstance(rope, dict):
             return
         elif len(rope) == 2:
@@ -479,16 +510,21 @@ class PlotHandler:
         return max(p.y_max() for p in self.plotters.values())
 
 
+# pylint: disable=too-many-instance-attributes
 class VarHandler:
     """Handle individual variable logic."""
 
-    def __init__(self, var_name, data, y_start, model_names, combined, colors):
+    def __init__(
+        self, var_name, data, y_start, model_names, combined, combine_dims, colors, labeller
+    ):
         self.var_name = var_name
         self.data = data
         self.y_start = y_start
         self.model_names = model_names
         self.combined = combined
+        self.combine_dims = combine_dims
         self.colors = colors
+        self.labeller = labeller
         self.model_color = dict(zip(self.model_names, self.colors))
         max_chains = max(datum.chain.max().values for datum in data)
         self.chain_offset = len(data) * 0.45 / max(1, max_chains)
@@ -499,10 +535,10 @@ class VarHandler:
         """Iterate over models and chains for each variable."""
         if self.combined:
             grouped_data = [[(0, datum)] for datum in self.data]
-            skip_dims = {"chain"}
+            skip_dims = self.combine_dims.union({"chain"})
         else:
             grouped_data = [datum.groupby("chain") for datum in self.data]
-            skip_dims = set()
+            skip_dims = self.combine_dims
 
         label_dict = OrderedDict()
         selection_list = []
@@ -515,15 +551,13 @@ class VarHandler:
                     reverse_selections=True,
                 )
                 datum_list = list(datum_iter)
-                for _, selection, values in datum_list:
+                for _, selection, isel, values in datum_list:
                     selection_list.append(selection)
-                    if not selection:
+                    if not selection or not len(selection_list) % len(datum_list):
                         var_name = self.var_name
-                    elif not len(selection_list) % len(datum_list):
-                        var_name = self.var_name + ":"
                     else:
                         var_name = ""
-                    label = make_label(var_name, selection, position="beside")
+                    label = self.labeller.make_label_flat(var_name, selection, isel)
                     if label not in label_dict:
                         label_dict[label] = OrderedDict()
                     if name not in label_dict[label]:
@@ -533,10 +567,7 @@ class VarHandler:
         y = self.y_start
         for idx, (label, model_data) in enumerate(label_dict.items()):
             for model_name, value_list in model_data.items():
-                if model_name:
-                    row_label = "{}: {}".format(model_name, label)
-                else:
-                    row_label = label
+                row_label = self.labeller.make_model_label(model_name, label)
                 for values in value_list:
                     yield y, row_label, label, selection_list[idx], values, self.model_color[
                         model_name
@@ -551,11 +582,12 @@ class VarHandler:
         for y, label, _, _, vals, color in self.iterator():
             y_ticks[label].append((y, vals, color))
         labels, ticks, vals, colors = [], [], [], []
-        for label, data in y_ticks.items():
-            labels.append(label)
-            ticks.append(np.mean([j[0] for j in data]))
-            vals.append(np.vstack([j[1] for j in data]))
-            colors.append(data[0][2])  # the colors are all the same
+        for label, all_data in y_ticks.items():
+            for data in all_data:
+                labels.append(label)
+                ticks.append(data[0])
+                vals.append(np.array(data[1]))
+                colors.append(data[2])  # the colors are all the same
         return labels, ticks, vals, colors
 
     def treeplot(self, qlist, hdi_prob):
