@@ -1,9 +1,11 @@
 """Bayesian p-value Posterior/Prior predictive plot."""
 import numpy as np
 
+from ..labels import BaseLabeller
 from ..rcparams import rcParams
 from ..utils import _var_names
-from .plot_utils import default_grid, filter_plotters_list, get_plotting_function, xarray_var_iter
+from .plot_utils import default_grid, filter_plotters_list, get_plotting_function
+from ..sel_utils import xarray_var_iter
 
 
 def plot_bpv(
@@ -20,6 +22,7 @@ def plot_bpv(
     grid=None,
     figsize=None,
     textsize=None,
+    labeller=None,
     data_pairs=None,
     var_names=None,
     filter_vars=None,
@@ -39,7 +42,8 @@ def plot_bpv(
     Parameters
     ----------
     data : az.InferenceData object
-        InferenceData object containing the observed and posterior/prior predictive data.
+        :class:`arviz.InferenceData` object containing the observed and
+        posterior/prior predictive data.
     kind : str
         Type of plot to display ("p_value", "u_value", "t_stat"). Defaults to u_value.
         For "p_value" we compute p := p(y* ≤ y | y). This is the probability of the data y being
@@ -49,17 +53,18 @@ def plot_bpv(
         This is also known as marginal p_value. The ideal distribution is uniform. This is similar
         to the LOO-pit calculation/plot, the difference is than in LOO-pit plot we compute
         pi = p(yi* r ≤ yi | y-i ), where y-i, is all other data except yi.
-        For "t_stat" we compute := p(T(y)* ≤ T(y) | y) where T is any T statistic. See t_stat
+        For "t_stat" we compute := p(T(y)* ≤ T(y) | y) where T is any test statistic. See t_stat
         argument below for details of available options.
     t_stat : str, float, or callable
-        T statistics to compute from the observations and predictive distributions. Allowed strings
-        are "mean", "median" or "std". Defaults to "median". Alternative a quantile can be passed
-        as a float (or str) in the interval (0, 1). Finally a user defined function is also
+        Test statistics to compute from the observations and predictive distributions.
+        Allowed strings are "mean", "median" or "std". Defaults to "median".
+        Alternative a quantile can be passed as a float (or str) in the
+        interval (0, 1). Finally a user defined function is also
         acepted, see examples section for details.
     bpv : bool
-        If True (default) add the bayesian p_value to the legend when kind = t_stat.
+        If True (default) add the Bayesian p_value to the legend when ``kind = t_stat``.
     plot_mean : bool
-        Whether or not to plot the mean T statistic. Defaults to True.
+        Whether or not to plot the mean test statistic. Defaults to True.
     reference : str
         How to compute the distributions used as reference for u_values or p_values. Allowed values
         are "analytical" (default) and "samples". Use `None` to do not plot any reference.
@@ -68,7 +73,7 @@ def plot_bpv(
         Show scaled mean square error between uniform distribution and marginal p_value
         distribution. Defaults to False.
     n_ref : int, optional
-        Number of reference distributions to sample when `reference=samples`. Defaults to 100.
+        Number of reference distributions to sample when ``reference=samples``. Defaults to 100.
     hdi_prob: float, optional
         Probability for the highest density interval for the analytical reference distribution when
         computing u_values. Should be in the interval (0, 1]. Defaults to
@@ -82,7 +87,7 @@ def plot_bpv(
         Figure size. If None it will be defined automatically.
     textsize : float
         Text size scaling factor for labels, titles and lines. If None it will be
-        autoscaled based on figsize.
+        autoscaled based on ``figsize``.
     data_pairs : dict
         Dictionary containing relations between observed data and posterior/prior predictive data.
         Dictionary structure:
@@ -90,17 +95,20 @@ def plot_bpv(
         - key = data var_name
         - value = posterior/prior predictive var_name
 
-        For example, `data_pairs = {'y' : 'y_hat'}`
+        For example, ``data_pairs = {'y' : 'y_hat'}``
         If None, it will assume that the observed data and the posterior/prior
         predictive data have the same variable name.
+    labeller : labeller instance, optional
+        Class providing the method ``make_pp_label`` to generate the labels in the plot titles.
+        Read the :ref:`label_guide` for more details and usage examples.
     var_names : list of variable names
-        Variables to be plotted, if `None` all variable are plotted. Prefix the variables by `~`
+        Variables to be plotted, if `None` all variable are plotted. Prefix the variables by ``~``
         when you want to exclude them from the plot.
     filter_vars : {None, "like", "regex"}, optional, default=None
         If `None` (default), interpret var_names as the real variables names. If "like",
         interpret var_names as substrings of the real variables names. If "regex",
         interpret var_names as regular expressions on the real variables names. A la
-        `pandas.filter`.
+        ``pandas.filter``.
     coords : dict
         Dictionary mapping dimensions to selected coordinates to be plotted.
         Dimensions without a mapping specified will include all coordinates for
@@ -113,7 +121,7 @@ def plot_bpv(
         List of dimensions to flatten in posterior_predictive/prior_predictive. Only flattens
         across the coordinates specified in the coords argument. Defaults to flattening all
         of the dimensions. Dimensions should match flatten excluding dimensions for data_pairs
-        parameters. If flatten is defined and flatten_pp is None, then `flatten_pp=flatten`.
+        parameters. If flatten is defined and flatten_pp is None, then ``flatten_pp=flatten``.
     legend : bool
         Add legend to figure. By default True.
     ax : numpy array-like of matplotlib axes or bokeh figures, optional
@@ -122,10 +130,14 @@ def plot_bpv(
     backend : str, optional
         Select plotting backend {"matplotlib","bokeh"}. Default "matplotlib".
     plot_ref_kwargs :  dict, optional
-        Extra keyword arguments to control how reference is represented. Passed to `plt.plot` or
-        `plt.axhspan`(when `kind=u_value` and `reference=analytical`).
+        Extra keyword arguments to control how reference is represented.
+        Passed to :meth:`matplotlib.axes.Axes.plot` or
+        :meth:`matplotlib.axes.Axes.axhspan` (when ``kind=u_value``
+        and ``reference=analytical``).
     backend_kwargs : bool, optional
-        These are kwargs specific to the backend being used. For additional documentation
+        These are kwargs specific to the backend being used, passed to
+        :func:`matplotlib.pyplot.subplots` or
+        :func:`bokeh.plotting.figure`. For additional documentation
         check the plotting method of the backend.
     group : {"prior", "posterior"}, optional
         Specifies which InferenceData group should be plotted. Defaults to 'posterior'.
@@ -137,6 +149,12 @@ def plot_bpv(
     -------
     axes: matplotlib axes or bokeh figures
 
+    See Also
+    --------
+    plot_ppc : Plot for posterior/prior predictive checks.
+    plot_loo_pit : Plot Leave-One-Out probability integral transformation (PIT) predictive checks.
+    plot_dist_comparison : Plot to compare fitted and unfitted distributions.
+
     References
     ----------
     * Gelman et al. (2013) see http://www.stat.columbia.edu/~gelman/book/ pages 151-153 for details
@@ -152,7 +170,7 @@ def plot_bpv(
         >>> data = az.load_arviz_data("regression1d")
         >>> az.plot_bpv(data, kind="p_value")
 
-    Plot custom t statistic comparison.
+    Plot custom test statistic comparison.
 
     .. plot::
         :context: close-figs
@@ -164,9 +182,9 @@ def plot_bpv(
     if group not in ("posterior", "prior"):
         raise TypeError("`group` argument must be either `posterior` or `prior`")
 
-    for groups in ("{}_predictive".format(group), "observed_data"):
+    for groups in (f"{group}_predictive", "observed_data"):
         if not hasattr(data, groups):
-            raise TypeError('`data` argument must have the group "{group}"'.format(group=groups))
+            raise TypeError(f'`data` argument must have the group "{groups}"')
 
     if kind.lower() not in ("t_stat", "u_value", "p_value"):
         raise TypeError("`kind` argument must be either `t_stat`, `u_value`, or `p_value`")
@@ -186,6 +204,9 @@ def plot_bpv(
     if data_pairs is None:
         data_pairs = {}
 
+    if labeller is None:
+        labeller = BaseLabeller()
+
     if backend is None:
         backend = rcParams["plot.backend"]
     backend = backend.lower()
@@ -260,6 +281,7 @@ def plot_bpv(
         color=color,
         figsize=figsize,
         textsize=textsize,
+        labeller=labeller,
         plot_ref_kwargs=plot_ref_kwargs,
         backend_kwargs=backend_kwargs,
         show=show,
