@@ -5,14 +5,11 @@ from typing import List, Optional, Union
 import numpy as np
 
 from ..data import convert_to_dataset
+from ..labels import BaseLabeller
+from ..sel_utils import xarray_to_ndarray, xarray_var_iter
+from .plot_utils import get_plotting_function
 from ..rcparams import rcParams
 from ..utils import _var_names, get_coords
-from .plot_utils import (
-    get_plotting_function,
-    xarray_to_ndarray,
-    xarray_var_iter,
-    make_label,
-)
 
 
 def plot_pair(
@@ -20,6 +17,7 @@ def plot_pair(
     group="posterior",
     var_names: Optional[List[str]] = None,
     filter_vars: Optional[str] = None,
+    combine_dims=None,
     coords=None,
     marginals=False,
     figsize=None,
@@ -31,6 +29,7 @@ def plot_pair(
     fill_last=False,
     divergences=False,
     colorbar=False,
+    labeller=None,
     ax=None,
     divergences_kwargs=None,
     scatter_kwargs=None,
@@ -52,33 +51,36 @@ def plot_pair(
     Parameters
     ----------
     data: obj
-        Any object that can be converted to an az.InferenceData object
-        Refer to documentation of az.convert_to_dataset for details
+        Any object that can be converted to an :class:`arviz.InferenceData` object.
+        Refer to documentation of :func:`arviz.convert_to_dataset` for details
     group: str, optional
         Specifies which InferenceData group should be plotted.  Defaults to 'posterior'.
     var_names: list of variable names, optional
         Variables to be plotted, if None all variable are plotted. Prefix the
-        variables by `~` when you want to exclude them from the plot.
+        variables by ``~`` when you want to exclude them from the plot.
     filter_vars: {None, "like", "regex"}, optional, default=None
         If `None` (default), interpret var_names as the real variables names. If "like",
         interpret var_names as substrings of the real variables names. If "regex",
         interpret var_names as regular expressions on the real variables names. A la
-        `pandas.filter`.
+        ``pandas.filter``.
+    combine_dims : set_like of str, optional
+        List of dimensions to reduce. Defaults to reducing only the "chain" and "draw" dimensions.
+        See the :ref:`this section <common_combine_dims>` for usage examples.
     coords: mapping, optional
-        Coordinates of var_names to be plotted. Passed to `Dataset.sel`
+        Coordinates of var_names to be plotted. Passed to :meth:`xarray.Dataset.sel`.
     marginals: bool, optional
         If True pairplot will include marginal distributions for every variable
     figsize: figure size tuple
         If None, size is (8 + numvars, 8 + numvars)
     textsize: int
-        Text size for labels. If None it will be autoscaled based on figsize.
+        Text size for labels. If None it will be autoscaled based on ``figsize``.
     kind : str or List[str]
         Type of plot to display (scatter, kde and/or hexbin)
     gridsize: int or (int, int), optional
-        Only works for kind=hexbin.
-        The number of hexagons in the x-direction. The corresponding number of hexagons in the
-        y-direction is chosen such that the hexagons are approximately regular.
-        Alternatively, gridsize can be a tuple with two elements specifying the number of hexagons
+        Only works for ``kind=hexbin``. The number of hexagons in the x-direction.
+        The corresponding number of hexagons in the y-direction is chosen
+        such that the hexagons are approximately regular. Alternatively, gridsize
+        can be a tuple with two elements specifying the number of hexagons
         in the x-direction and the y-direction.
     contour : bool, optional, deprecated, Defaults to True.
         If True plot the 2D KDE using contours, otherwise plot a smooth 2D KDE. Defaults to True.
@@ -90,37 +92,47 @@ def plot_pair(
         or 'posterior'.
     colorbar: bool
         If True a colorbar will be included as part of the plot (Defaults to False).
-        Only works when kind=hexbin
+        Only works when ``kind=hexbin``
+    labeller : labeller instance, optional
+        Class providing the method ``make_label_vert`` to generate the labels in the plot.
+        Read the :ref:`label_guide` for more details and usage examples.
     ax: axes, optional
         Matplotlib axes or bokeh figures.
     divergences_kwargs: dicts, optional
-        Additional keywords passed to ax.scatter for divergences
+        Additional keywords passed to :meth:`matplotlib.axes.Axes.scatter` for divergences
     scatter_kwargs:
-        Additional keywords passed to ax.plot when using scatter kind
+        Additional keywords passed to :meth:`matplotlib.axes.Axes.plot` when using scatter kind
     kde_kwargs: dict, optional
-        Additional keywords passed to az.plot_kde when using kde kind
+        Additional keywords passed to :func:`arviz.plot_kde` when using kde kind
     hexbin_kwargs: dict, optional
-        Additional keywords passed to ax.hexbin when using hexbin kind
+        Additional keywords passed to :meth:`matplotlib.axes.Axes.hexbin` when
+        using hexbin kind
     backend: str, optional
         Select plotting backend {"matplotlib","bokeh"}. Default "matplotlib".
     backend_kwargs: bool, optional
-        These are kwargs specific to the backend being used. For additional documentation
-        check the plotting method of the backend.
+        These are kwargs specific to the backend being used, passed to
+        :func:`matplotlib.pyplot.subplots` or
+        :func:`bokeh.plotting.figure`.
     marginal_kwargs: dict, optional
-        Additional keywords passed to az.plot_dist, modifying the marginal distributions
-        plotted in the diagonal.
+        Additional keywords passed to :func:`arviz.plot_dist`, modifying the
+        marginal distributions plotted in the diagonal.
     point_estimate: str, optional
         Select point estimate from 'mean', 'mode' or 'median'. The point estimate will be
         plotted using a scatter marker and vertical/horizontal lines.
     point_estimate_kwargs: dict, optional
-        Additional keywords passed to ax.vline, ax.hline (matplotlib) or ax.square, Span (bokeh)
+        Additional keywords passed to :meth:`matplotlib.axes.Axes.axvline`,
+        :meth:`matplotlib.axes.Axes.axhline` (matplotlib) or
+        :class:`bokeh:bokeh.models.Span` (bokeh)
     point_estimate_marker_kwargs: dict, optional
-        Additional keywords passed to ax.scatter in point estimate plot. Not available in bokeh
+        Additional keywords passed to :meth:`matplotlib.axes.Axes.scatter`
+        or :meth:`bokeh:bokeh.plotting.Figure.square` in point
+        estimate plot. Not available in bokeh
     reference_values: dict, optional
         Reference values for the plotted variables. The Reference values will be plotted
         using a scatter marker
     reference_values_kwargs: dict, optional
-        Additional keywords passed to ax.plot or ax.circle in reference values plot
+        Additional keywords passed to :meth:`matplotlib.axes.Axes.plot` or
+        :meth:`bokeh:bokeh.plotting.Figure.circle` in reference values plot
     show: bool, optional
         Call backend show function.
 
@@ -128,6 +140,11 @@ def plot_pair(
     -------
     axes: matplotlib axes or bokeh figures
 
+    See Also
+    --------
+    plot_joint : Plot a scatter or hexbin of two variables with their
+                 respective marginals distributions.
+
     Examples
     --------
     KDE Pair Plot
@@ -175,7 +192,7 @@ def plot_pair(
     else:
         kind_boolean = [kind[i] in valid_kinds for i in range(len(kind))]
     if not np.all(kind_boolean):
-        raise ValueError((f"Plot type {kind} not recognized." "Plot type must be in {valid_kinds}"))
+        raise ValueError(f"Plot type {kind} not recognized. Plot type must be in {valid_kinds}")
     if fill_last or contour:
         warnings.warn(
             "fill_last and contour will be deprecated. Please use kde_kwargs",
@@ -191,13 +208,20 @@ def plot_pair(
     if coords is None:
         coords = {}
 
+    if labeller is None:
+        labeller = BaseLabeller()
+
     # Get posterior draws and combine chains
     dataset = convert_to_dataset(data, group=group)
     var_names = _var_names(var_names, dataset, filter_vars)
     plotters = list(
-        xarray_var_iter(get_coords(dataset, coords), var_names=var_names, combined=True)
+        xarray_var_iter(
+            get_coords(dataset, coords), var_names=var_names, skip_dims=combine_dims, combined=True
+        )
     )
-    flat_var_names = [make_label(var_name, selection) for var_name, selection, _ in plotters]
+    flat_var_names = [
+        labeller.make_label_vert(var_name, sel, isel) for var_name, sel, isel, _ in plotters
+    ]
 
     divergent_data = None
     diverging_mask = None
