@@ -36,10 +36,9 @@ def _bw_silverman(x, x_std=None, **kwargs):  # pylint: disable=unused-argument
 def _bw_isj(x, grid_counts=None, x_std=None, x_range=None):
     """Improved Sheather-Jones bandwidth estimation.
 
-    Improved Sheather and Jones method as explained in [1]_.
-    This is an internal version pretended to be used by the KDE estimator.
-    When used internally computation time is saved because things like minimums,
-    maximums and the grid are pre-computed.
+    Improved Sheather and Jones method as explained in [1]_. This method is used internally by the
+    KDE estimator, resulting in saved computation time as minimums, maximums and the grid are
+    pre-computed.
 
     References
     ----------
@@ -72,7 +71,7 @@ def _bw_isj(x, grid_counts=None, x_std=None, x_range=None):
     a_sq = a_k[range(1, grid_len)] ** 2
 
     t = _root(_fixed_point, x_len, args=(x_len, k_sq, a_sq), x=x)
-    h = t ** 0.5 * x_range
+    h = t**0.5 * x_range
     return h
 
 
@@ -86,14 +85,12 @@ def _bw_experimental(x, grid_counts=None, x_std=None, x_range=None):
 def _bw_taylor(x):
     """Taylor's rule for circular bandwidth estimation.
 
-    This function implements a rule-of-thumb for choosing the bandwidth of
-    a von Mises kernel density estimator that assumes the underlying
-    distribution is von Mises as introduced in [1]_.
+    This function implements a rule-of-thumb for choosing the bandwidth of a von Mises kernel
+    density estimator that assumes the underlying distribution is von Mises as introduced in [1]_.
     It is analogous to Scott's rule for the Gaussian KDE.
 
-    Circular bandwidth has a different scale from linear bandwidth.
-    Unlike linear scale, low bandwidths are associated with oversmoothing
-    while high values are associated with undersmoothing.
+    Circular bandwidth has a different scale from linear bandwidth. Unlike linear scale, low
+    bandwidths are associated with oversmoothing and high values with undersmoothing.
 
     References
     ----------
@@ -103,8 +100,8 @@ def _bw_taylor(x):
     """
     x_len = len(x)
     kappa = _kappa_mle(x)
-    num = 3 * x_len * kappa ** 2 * ive(2, 2 * kappa)
-    den = 4 * np.pi ** 0.5 * ive(0, kappa) ** 2
+    num = 3 * x_len * kappa**2 * ive(2, 2 * kappa)
+    den = 4 * np.pi**0.5 * ive(0, kappa) ** 2
     return (num / den) ** 0.4
 
 
@@ -140,7 +137,7 @@ def _get_bw(x, bw, grid_counts=None, x_std=None, x_range=None):
             (
                 "`bw` must not be of type `bool`.\n"
                 "Expected a positive numeric or one of the following strings:\n"
-                f"{list(_BW_METHODS_LINEAR.keys())}."
+                f"{list(_BW_METHODS_LINEAR)}."
             )
         )
     if isinstance(bw, (int, float)):
@@ -149,11 +146,11 @@ def _get_bw(x, bw, grid_counts=None, x_std=None, x_range=None):
     elif isinstance(bw, str):
         bw_lower = bw.lower()
 
-        if bw_lower not in _BW_METHODS_LINEAR.keys():
+        if bw_lower not in _BW_METHODS_LINEAR:
             raise ValueError(
                 "Unrecognized bandwidth method.\n"
                 f"Input is: {bw_lower}.\n"
-                f"Expected one of: {list(_BW_METHODS_LINEAR.keys())}."
+                f"Expected one of: {list(_BW_METHODS_LINEAR)}."
             )
 
         bw_fun = _BW_METHODS_LINEAR[bw_lower]
@@ -162,7 +159,7 @@ def _get_bw(x, bw, grid_counts=None, x_std=None, x_range=None):
         raise ValueError(
             "Unrecognized `bw` argument.\n"
             "Expected a positive numeric or one of the following strings:\n"
-            f"{list(_BW_METHODS_LINEAR.keys())}."
+            f"{list(_BW_METHODS_LINEAR)}."
         )
     return bw
 
@@ -184,11 +181,11 @@ def _a1inv(x):
     Returns the value k, such that a1inv(x) = k, i.e. a1(k) = x.
     """
     if 0 <= x < 0.53:
-        return 2 * x + x ** 3 + (5 * x ** 5) / 6
+        return 2 * x + x**3 + (5 * x**5) / 6
     elif x < 0.85:
         return -0.4 + 1.39 * x + 0.43 / (1 - x)
     else:
-        return 1 / (x ** 3 - 4 * x ** 2 + 3 * x)
+        return 1 / (x**3 - 4 * x**2 + 3 * x)
 
 
 def _kappa_mle(x):
@@ -238,7 +235,7 @@ def _fixed_point(t, N, k_sq, a_sq):
     a_sq = np.asfarray(a_sq, dtype=np.float64)
 
     l = 7
-    f = np.sum(np.power(k_sq, l) * a_sq * np.exp(-k_sq * np.pi ** 2 * t))
+    f = np.sum(np.power(k_sq, l) * a_sq * np.exp(-k_sq * np.pi**2 * t))
     f *= 0.5 * np.pi ** (2.0 * l)
 
     for j in np.arange(l - 1, 2 - 1, -1):
@@ -246,10 +243,10 @@ def _fixed_point(t, N, k_sq, a_sq):
         c2 = np.product(np.arange(1.0, 2 * j + 1, 2, dtype=np.float64))
         c2 /= (np.pi / 2) ** 0.5
         t_j = np.power((c1 * (c2 / (N * f))), (2.0 / (3.0 + 2.0 * j)))
-        f = np.sum(k_sq ** j * a_sq * np.exp(-k_sq * np.pi ** 2.0 * t_j))
+        f = np.sum(k_sq**j * a_sq * np.exp(-k_sq * np.pi**2.0 * t_j))
         f *= 0.5 * np.pi ** (2 * j)
 
-    out = t - (2 * N * np.pi ** 0.5 * f) ** (-0.4)
+    out = t - (2 * N * np.pi**0.5 * f) ** (-0.4)
     return out
 
 
@@ -268,51 +265,11 @@ def _root(function, N, args, x):
             tol *= 2.0
             found = False
         if bw <= 0 or tol >= 1:
-            # warnings.warn(
-            #    "Improved Sheather-Jones did not converge as expected. "
-            #    "Using Silverman's rule instead.",
-            #    Warning
-            # )
             bw = (_bw_silverman(x) / np.ptp(x)) ** 2
             return bw
     return bw
 
 
-def _check_type(x):
-    """Check the input is of the correct type.
-
-    It only accepts numeric lists/numpy arrays of 1 dimension or something that
-    can be flattened to 1 dimension.
-
-    Parameters
-    ----------
-    x : Object whose type is checked before computing the KDE.
-
-    Returns
-    -------
-    x : 1-D numpy array
-        If no error is thrown, a 1 dimensional array of
-        sample data from the variable for which a density estimate is desired.
-    """
-    # Will raise an error if `x` can't be casted to numeric or flattened to one dimension.
-    try:
-        x = np.asfarray(x).flatten()
-    except Exception as e:
-        warnings.warn(
-            "The following exception occurred while trying to convert `x`"
-            "to a 1 dimensional float array."
-        )
-        raise e
-
-    x = x[np.isfinite(x)]
-
-    if x.size == 0:
-        raise ValueError("`x` does not contain any finite number.")
-    if x.size == 1:
-        raise ValueError("`x` is of length 1. Can't produce a KDE with only one data point.")
-    return x
-
-
 def _check_custom_lims(custom_lims, x_min, x_max):
     """Check if `custom_lims` are of the correct type.
 
@@ -401,9 +358,7 @@ def _get_grid(
         Maximum value of the grid
     """
     # Set up number of bins.
-    if grid_len < 100:
-        grid_len = 100
-    grid_len = int(grid_len)
+    grid_len = max(int(grid_len), 100)
 
     # Set up domain
     if custom_lims is not None:
@@ -423,107 +378,123 @@ def _get_grid(
 def kde(x, circular=False, **kwargs):
     """One dimensional density estimation.
 
-    It is a wrapper around `kde_linear()` and `kde_circular()`.
+    It is a wrapper around ``kde_linear()`` and ``kde_circular()``.
 
     Parameters
     ----------
-    x : 1D numpy array
+    x: 1D numpy array
         Data used to calculate the density estimation.
-        Theoritically it is a random sample obtained from $f$,
-        the true probability density function we aim to estimate.
     circular: bool, optional
-        Whether `x` is a circular variable or not. Defaults to False.
-    **kwargs: Arguments passed to `kde_linear()` and `kde_circular()`.
+        Whether ``x`` is a circular variable or not. Defaults to False.
+    **kwargs
+        Arguments passed to ``kde_linear()`` and ``kde_circular()``.
         See their documentation for more info.
 
     Returns
     -------
-    grid : Gridded numpy array for the x values.
-    pdf : Numpy array for the density estimates.
+    grid: Gridded numpy array for the x values.
+    pdf: Numpy array for the density estimates.
     bw: optional, the estimated bandwidth.
 
     Examples
     --------
     Default density estimation for linear data
+
     .. plot::
         :context: close-figs
 
-    >>> import numpy as np
-    >>> import matplotlib.pyplot as plt
-    >>> from arviz import kde
-    >>>
-    >>> rvs = np.random.gamma(shape=1.8, size=1000)
-    >>> grid, pdf = kde(rvs)
-    >>> plt.plot(grid, pdf)
-    >>> plt.show()
+        >>> import numpy as np
+        >>> import matplotlib.pyplot as plt
+        >>> from arviz import kde
+        >>>
+        >>> rvs = np.random.gamma(shape=1.8, size=1000)
+        >>> grid, pdf = kde(rvs)
+        >>> plt.plot(grid, pdf)
+        >>> plt.show()
 
     Density estimation for linear data with Silverman's rule bandwidth
+
     .. plot::
         :context: close-figs
 
-    >>> grid, pdf = kde(rvs, bw="silverman")
-    >>> plt.plot(grid, pdf)
-    >>> plt.show()
+        >>> grid, pdf = kde(rvs, bw="silverman")
+        >>> plt.plot(grid, pdf)
+        >>> plt.show()
 
     Density estimation for linear data with scaled bandwidth
+
     .. plot::
         :context: close-figs
 
-    >>> # bw_fct > 1 means more smoothness.
-    >>> grid, pdf = kde(rvs, bw_fct=2.5)
-    >>> plt.plot(grid, pdf)
-    >>> plt.show()
+        >>> # bw_fct > 1 means more smoothness.
+        >>> grid, pdf = kde(rvs, bw_fct=2.5)
+        >>> plt.plot(grid, pdf)
+        >>> plt.show()
 
     Default density estimation for linear data with extended limits
+
     .. plot::
         :context: close-figs
 
-    >>> grid, pdf = kde(rvs, bound_correction=False, extend=True, extend_fct=0.5)
-    >>> plt.plot(grid, pdf)
-    >>> plt.show()
+        >>> grid, pdf = kde(rvs, bound_correction=False, extend=True, extend_fct=0.5)
+        >>> plt.plot(grid, pdf)
+        >>> plt.show()
 
     Default density estimation for linear data with custom limits
+
     .. plot::
         :context: close-figs
-    # It accepts tuples and lists of length 2.
-    >>> grid, pdf = kde(rvs, bound_correction=False, custom_lims=(0, 10))
-    >>> plt.plot(grid, pdf)
-    >>> plt.show()
+
+        >>> # It accepts tuples and lists of length 2.
+        >>> grid, pdf = kde(rvs, bound_correction=False, custom_lims=(0, 10))
+        >>> plt.plot(grid, pdf)
+        >>> plt.show()
 
     Default density estimation for circular data
+
     .. plot::
         :context: close-figs
 
-    >>> rvs = np.random.vonmises(mu=np.pi, kappa=1, size=500)
-    >>> grid, pdf = kde(rvs, circular=True)
-    >>> plt.plot(grid, pdf)
-    >>> plt.show()
+        >>> rvs = np.random.vonmises(mu=np.pi, kappa=1, size=500)
+        >>> grid, pdf = kde(rvs, circular=True)
+        >>> plt.plot(grid, pdf)
+        >>> plt.show()
 
     Density estimation for circular data with scaled bandwidth
+
     .. plot::
         :context: close-figs
 
-    >>> rvs = np.random.vonmises(mu=np.pi, kappa=1, size=500)
-    >>> # bw_fct > 1 means less smoothness.
-    >>> grid, pdf = kde(rvs, circular=True, bw_fct=3)
-    >>> plt.plot(grid, pdf)
-    >>> plt.show()
+        >>> rvs = np.random.vonmises(mu=np.pi, kappa=1, size=500)
+        >>> # bw_fct > 1 means less smoothness.
+        >>> grid, pdf = kde(rvs, circular=True, bw_fct=3)
+        >>> plt.plot(grid, pdf)
+        >>> plt.show()
 
     Density estimation for circular data with custom limits
+
     .. plot::
         :context: close-figs
-    >>> # This is still experimental, does not always work.
-    >>> rvs = np.random.vonmises(mu=0, kappa=30, size=500)
-    >>> grid, pdf = kde(rvs, circular=True, custom_lims=(-1, 1))
-    >>> plt.plot(grid, pdf)
-    >>> plt.show()
+
+        >>> # This is still experimental, does not always work.
+        >>> rvs = np.random.vonmises(mu=0, kappa=30, size=500)
+        >>> grid, pdf = kde(rvs, circular=True, custom_lims=(-1, 1))
+        >>> plt.plot(grid, pdf)
+        >>> plt.show()
 
     See Also
     --------
     plot_kde : Compute and plot a kernel density estimate.
-    arviz.stats.density_utils.kde: Arviz KDE estimator
     """
+    x = x[np.isfinite(x)]
+    if x.size == 0 or np.all(x == x[0]):
+        warnings.warn("Your data appears to have a single value or no finite values")
+
+        return np.zeros(2), np.array([np.nan] * 2)
+
     if circular:
+        if circular == "degrees":
+            x = np.radians(x)
         kde_fun = _kde_circular
     else:
         kde_fun = _kde_linear
@@ -554,44 +525,39 @@ def _kde_linear(
     ----------
     x : 1D numpy array
         Data used to calculate the density estimation.
-        Theoritically it is a random sample obtained from $f$,
-        the true probability density function we aim to estimate.
     bw: int, float or str, optional
         If numeric, indicates the bandwidth and must be positive.
-        If str, indicates the method to estimate the bandwidth and must be
-        one of "scott", "silverman", "isj" or "experimental".
-        Defaults to "experimental".
+        If str, indicates the method to estimate the bandwidth and must be one of "scott",
+        "silverman", "isj" or "experimental". Defaults to "experimental".
     adaptive: boolean, optional
-        Indicates if the bandwidth is adaptative or not.
-        It is the recommended approach when there are multiple modalities
-        with different spread.
+        Indicates if the bandwidth is adaptive or not.
+        It is the recommended approach when there are multiple modes with different spread.
         It is not compatible with convolution. Defaults to False.
     extend: boolean, optional
         Whether to extend the observed range for `x` in the estimation.
-        It extends each bound by a multiple of the standard deviation of `x`
-        given by `extend_fct`. Defaults to False.
+        It extends each bound by a multiple of the standard deviation of `x` given by `extend_fct`.
+        Defaults to False.
     bound_correction: boolean, optional
         Whether to perform boundary correction on the bounds of `x` or not.
         Defaults to True.
     extend_fct: float, optional
-        Number of standard deviations used to widen the
-        lower and upper bounds of `x`. Defaults to 0.5.
+        Number of standard deviations used to widen the lower and upper bounds of `x`.
+        Defaults to 0.5.
     bw_fct: float, optional
         A value that multiplies `bw` which enables tuning smoothness by hand.
-        Must be positive. Values below 1 decrease smoothness while values
-        above 1 decrease it. Defaults to 1 (no modification).
+        Must be positive. Values below 1 decrease smoothness while values above 1 decrease it.
+        Defaults to 1 (no modification).
     bw_return: bool, optional
-        Whether to return the estimated bandwidth in addition to the
-        other objects. Defaults to False.
+        Whether to return the estimated bandwidth in addition to the other objects.
+        Defaults to False.
     custom_lims: list or tuple, optional
-        A list or tuple of length 2 indicating custom bounds
-        for the range of `x`. Defaults to None which disables custom bounds.
+        A list or tuple of length 2 indicating custom bounds for the range of `x`.
+        Defaults to None which disables custom bounds.
     cumulative: bool, optional
         Whether return the PDF or the cumulative PDF. Defaults to False.
     grid_len: int, optional
-        The number of intervals used to bin the data points
-        (a.k.a. the length of the grid used in the estimation)
-        Defaults to 512.
+        The number of intervals used to bin the data points i.e. the length of the grid used in
+        the estimation. Defaults to 512.
 
     Returns
     -------
@@ -599,13 +565,6 @@ def _kde_linear(
     pdf : Numpy array for the density estimates.
     bw: optional, the estimated bandwidth.
     """
-    # Check `x` is from appropiate type
-    try:
-        x = _check_type(x)
-    except ValueError as e:
-        warnings.warn("Something failed: " + str(e))
-        return np.array([np.nan]), np.array([np.nan])
-
     # Check `bw_fct` is numeric and positive
     if not isinstance(bw_fct, (int, float, np.integer, np.floating)):
         raise TypeError(f"`bw_fct` must be a positive number, not an object of {type(bw_fct)}.")
@@ -614,10 +573,9 @@ def _kde_linear(
         raise ValueError(f"`bw_fct` must be a positive number, not {bw_fct}.")
 
     # Preliminary calculations
-    x_len = len(x)
     x_min = x.min()
     x_max = x.max()
-    x_std = (((x ** 2).sum() / x_len) - (x.sum() / x_len) ** 2) ** 0.5
+    x_std = np.std(x)
     x_range = x_max - x_min
 
     # Determine grid
@@ -656,44 +614,33 @@ def _kde_circular(
 ):
     """One dimensional density estimation for circular data.
 
-    Given an array of data points `x` measured in radians,
-    it returns an estimate of the probability density function that generated
-    the samples in `x`.
+    Given an array of data points `x` measured in radians, it returns an estimate of the
+    probability density function that generated the samples in `x`.
 
     Parameters
     ----------
     x : 1D numpy array
         Data used to calculate the density estimation.
-        Theoritically it is a random sample obtained from $f$,
-        the true probability density function we aim to estimate.
     bw: int, float or str, optional
         If numeric, indicates the bandwidth and must be positive.
-        If str, indicates the method to estimate the bandwidth and must be
-        "taylor" since it is the only option supported so far. Defaults to "taylor".
+        If str, indicates the method to estimate the bandwidth and must be "taylor" since it is the
+        only option supported so far. Defaults to "taylor".
     bw_fct: float, optional
-        A value that multiplies `bw` which enables tuning smoothness by hand.
-        Must be positive. Values above 1 decrease smoothness while values
-        below 1 decrease it. Defaults to 1 (no modification).
+        A value that multiplies `bw` which enables tuning smoothness by hand. Must be positive.
+        Values above 1 decrease smoothness while values below 1 decrease it.
+        Defaults to 1 (no modification).
     bw_return: bool, optional
-        Whether to return the estimated bandwidth in addition to the
-        other objects. Defaults to False.
+        Whether to return the estimated bandwidth in addition to the other objects.
+        Defaults to False.
     custom_lims: list or tuple, optional
-        A list or tuple of length 2 indicating custom bounds
-        for the range of `x`. Defaults to None which means the estimation
-        limits are [-pi, pi].
+        A list or tuple of length 2 indicating custom bounds for the range of `x`.
+        Defaults to None which means the estimation limits are [-pi, pi].
     cumulative: bool, optional
         Whether return the PDF or the cumulative PDF. Defaults to False.
     grid_len: int, optional
-        The number of intervals used to bin the data points
-        (a.k.a. the length of the grid used in the estimation)
-        Defaults to 512.
+        The number of intervals used to bin the data pointa i.e. the length of the grid used in the
+        estimation. Defaults to 512.
     """
-    try:
-        x = _check_type(x)
-    except ValueError as e:
-        warnings.warn("Something failed: " + str(e))
-        return np.array([np.nan]), np.array([np.nan])
-
     # All values between -pi and pi
     x = _normalize_angle(x)
 
@@ -751,9 +698,8 @@ def _kde_circular(
 def _kde_convolution(x, bw, grid_edges, grid_counts, grid_len, bound_correction, **kwargs):
     """Kernel density with convolution.
 
-    One dimensional Gaussian kernel density estimation via
-    convolution of the binned relative frequencies and a Gaussian filter.
-    This is an internal function used by `kde()`.
+    One dimensional Gaussian kernel density estimation via convolution of the binned relative
+    frequencies and a Gaussian filter. This is an internal function used by `kde()`.
     """
     # Calculate relative frequencies per bin
     bin_width = grid_edges[1] - grid_edges[0]
@@ -766,10 +712,6 @@ def _kde_convolution(x, bw, grid_edges, grid_counts, grid_len, bound_correction,
 
     grid = (grid_edges[1:] + grid_edges[:-1]) / 2
 
-    if not np.isfinite(bw):
-        warnings.warn("Something failed when estimating the bandwidth. Please check your data")
-        bw = 0
-
     kernel_n = int(bw * 2 * np.pi)
     if kernel_n == 0:
         kernel_n = 1
@@ -791,10 +733,9 @@ def _kde_convolution(x, bw, grid_edges, grid_counts, grid_len, bound_correction,
 def _kde_adaptive(x, bw, grid_edges, grid_counts, grid_len, bound_correction, **kwargs):
     """Compute Adaptive Kernel Density Estimation.
 
-    One dimensional adaptive Gaussian kernel density estimation.
-    The implementation uses the binning technique.
-    Since there is not an unique `bw`, the convolution is not possible.
-    The alternative implemented in this function is known as Abramson's method.
+    One dimensional adaptive Gaussian kernel density estimation. The implementation uses the binning
+    technique. Since there is not an unique `bw`, the convolution is not possible. The alternative
+    implemented in this function is known as Abramson's method.
     This is an internal function used by `kde()`.
     """
     # Pilot computations used for bandwidth adjustment
@@ -836,13 +777,13 @@ def _kde_adaptive(x, bw, grid_edges, grid_counts, grid_len, bound_correction, **
             [bw_adj[grid_npad - 1 :: -1], bw_adj, bw_adj[grid_len : grid_len - grid_npad - 1 : -1]]
         )
         pdf_mat = (grid_padded - grid_padded[:, None]) / bw_adj[:, None]
-        pdf_mat = np.exp(-0.5 * pdf_mat ** 2) * grid_counts[:, None]
+        pdf_mat = np.exp(-0.5 * pdf_mat**2) * grid_counts[:, None]
         pdf_mat /= (2 * np.pi) ** 0.5 * bw_adj[:, None]
         pdf = np.sum(pdf_mat[:, grid_npad : grid_npad + grid_len], axis=0) / len(x)
 
     else:
         pdf_mat = (grid - grid[:, None]) / bw_adj[:, None]
-        pdf_mat = np.exp(-0.5 * pdf_mat ** 2) * grid_counts[:, None]
+        pdf_mat = np.exp(-0.5 * pdf_mat**2) * grid_counts[:, None]
         pdf_mat /= (2 * np.pi) ** 0.5 * bw_adj[:, None]
         pdf = np.sum(pdf_mat, axis=0) / len(x)
 
@@ -862,7 +803,8 @@ def _fast_kde_2d(x, y, gridsize=(128, 128), circular=False):
     gridsize : tuple
         Number of points used to discretize data. Use powers of 2 for fft optimization
     circular: bool
-        If True, use circular boundaries. Defaults to False
+        If True use circular boundaries. Defaults to False
+
     Returns
     -------
     grid: A gridded 2D KDE of the input points (x, y)
@@ -896,7 +838,7 @@ def _fast_kde_2d(x, y, gridsize=(128, 128), circular=False):
     std_devs = np.diag(cov) ** 0.5
     kern_nx, kern_ny = np.round(scotts_factor * 2 * np.pi * std_devs)
 
-    inv_cov = np.linalg.inv(cov * scotts_factor ** 2)
+    inv_cov = np.linalg.inv(cov * scotts_factor**2)
 
     x_x = np.arange(kern_nx) - kern_nx / 2
     y_y = np.arange(kern_ny) - kern_ny / 2
@@ -912,8 +854,8 @@ def _fast_kde_2d(x, y, gridsize=(128, 128), circular=False):
     grid = coo_matrix((weights, xyi), shape=(n_x, n_y)).toarray()
     grid = convolve2d(grid, kernel, mode="same", boundary=boundary)
 
-    norm_factor = np.linalg.det(2 * np.pi * cov * scotts_factor ** 2)
-    norm_factor = len_x * d_x * d_y * norm_factor ** 0.5
+    norm_factor = np.linalg.det(2 * np.pi * cov * scotts_factor**2)
+    norm_factor = len_x * d_x * d_y * norm_factor**0.5
 
     grid /= norm_factor
 
@@ -935,7 +877,7 @@ def get_bins(values):
 
     Notes
     -----
-    Computes the width of the bins by taking the maximun of the Sturges and the Freedman-Diaconis
+    Computes the width of the bins by taking the maximum of the Sturges and the Freedman-Diaconis
     estimators. According to numpy `np.histogram` this provides good all around performance.
 
     The Sturges is a very simplistic estimator based on the assumption of normality of the data.
@@ -943,12 +885,18 @@ def get_bins(values):
     large data sets. The estimate depends only on size of the data.
 
     The Freedman-Diaconis rule uses interquartile range (IQR) to estimate the binwidth.
-    It is considered a robusts version of the Scott rule as the IQR is less affected by outliers
+    It is considered a robust version of the Scott rule as the IQR is less affected by outliers
     than the standard deviation. However, the IQR depends on fewer points than the standard
     deviation, so it is less accurate, especially for long tailed distributions.
     """
-    x_min = values.min().astype(int)
-    x_max = values.max().astype(int)
+    dtype = values.dtype.kind
+
+    if dtype == "i":
+        x_min = values.min().astype(int)
+        x_max = values.max().astype(int)
+    else:
+        x_min = values.min().astype(float)
+        x_max = values.max().astype(float)
 
     # Sturges histogram bin estimator
     bins_sturges = (x_max - x_min) / (np.log2(values.size) + 1)
@@ -957,9 +905,16 @@ def get_bins(values):
     iqr = np.subtract(*np.percentile(values, [75, 25]))  # pylint: disable=assignment-from-no-return
     bins_fd = 2 * iqr * values.size ** (-1 / 3)
 
-    width = np.round(np.max([1, bins_sturges, bins_fd])).astype(int)
+    if dtype == "i":
+        width = np.round(np.max([1, bins_sturges, bins_fd])).astype(int)
+        bins = np.arange(x_min, x_max + width + 1, width)
+    else:
+        width = np.max([bins_sturges, bins_fd])
+        if np.isclose(x_min, x_max):
+            width = 1e-3
+        bins = np.arange(x_min, x_max + width, width)
 
-    return np.arange(x_min, x_max + width + 1, width)
+    return bins
 
 
 def _sturges_formula(dataset, mult=1):
@@ -984,14 +939,11 @@ def _sturges_formula(dataset, mult=1):
     return int(np.ceil(mult * np.log2(dataset.draw.size)) + 1)
 
 
-def _circular_mean(x, na_rm=False):
+def _circular_mean(x):
     """Compute mean of circular variable measured in radians.
 
     The result is between -pi and pi.
     """
-    if na_rm:
-        x = x[~np.isnan(x)]
-
     sinr = np.sum(np.sin(x))
     cosr = np.sum(np.cos(x))
     mean = np.arctan2(sinr, cosr)
@@ -1002,8 +954,7 @@ def _circular_mean(x, na_rm=False):
 def _normalize_angle(x, zero_centered=True):
     """Normalize angles.
 
-    Take angles in radians and normalize them to [-pi, pi) or [0, 2 * pi)
-    depending on `zero_centered`.
+    Normalize angles in radians to [-pi, pi) or [0, 2 * pi) according to `zero_centered`.
     """
     if zero_centered:
         return (x + np.pi) % (2 * np.pi) - np.pi
@@ -1036,3 +987,34 @@ def histogram(data, bins, range_hist=None):
     hist, bin_edges = np.histogram(data, bins=bins, range=range_hist)
     hist_dens = hist / (hist.sum() * np.diff(bin_edges))
     return hist, hist_dens, bin_edges
+
+
+def _find_hdi_contours(density, hdi_probs):
+    """
+    Find contours enclosing regions of highest posterior density.
+
+    Parameters
+    ----------
+    density : array-like
+        A 2D KDE on a grid with cells of equal area.
+    hdi_probs : array-like
+        An array of highest density interval confidence probabilities.
+
+    Returns
+    -------
+    contour_levels : array
+        The contour levels corresponding to the given HDI probabilities.
+    """
+    # Using the algorithm from corner.py
+    sorted_density = np.sort(density, axis=None)[::-1]
+    sm = sorted_density.cumsum()
+    sm /= sm[-1]
+
+    contours = np.empty_like(hdi_probs)
+    for idx, hdi_prob in enumerate(hdi_probs):
+        try:
+            contours[idx] = sorted_density[sm <= hdi_prob][-1]
+        except IndexError:
+            contours[idx] = sorted_density[0]
+
+    return contours
