@@ -42,14 +42,16 @@ __all__ = [
     "loo",
     "loo_pit",
     "psislw",
+    "r2_samples",
     "r2_score",
     "summary",
     "waic",
+    "_calculate_ics",
 ]
 
 
 def compare(
-    dataset_dict: Mapping[str, InferenceData],
+    compare_dict: Mapping[str, InferenceData],
     ic: Optional[ICKeyword] = None,
     method: Literal["stacking", "BB-pseudo-BMA", "pseudo-MA"] = "stacking",
     b_samples: int = 1000,
@@ -67,8 +69,8 @@ def compare(
 
     Parameters
     ----------
-    dataset_dict: dict[str] -> InferenceData
-        A dictionary of model names and InferenceData objects
+    compare_dict: dict of {str: InferenceData or ELPDData}
+        A dictionary of model names and :class:`arviz.InferenceData` or ``ELPDData``.
     ic: str, optional
         Information Criterion (PSIS-LOO `loo` or WAIC `waic`) used to compare models. Defaults to
         ``rcParams["stats.information_criterion"]``.
@@ -93,7 +95,7 @@ def compare(
     seed: int or np.random.RandomState instance, optional
         If int or RandomState, use it for seeding Bayesian bootstrap. Only
         useful when method = 'BB-pseudo-BMA'. Default None the global
-        np.random state is used.
+        :mod:`numpy.random` state is used.
     scale: str, optional
         Output scale for IC. Available options are:
 
@@ -165,34 +167,12 @@ def compare(
         see https://doi.org/10.1007/s11222-016-9696-4
 
     """
-    names = list(dataset_dict.keys())
-    if scale is not None:
-        scale = cast(ScaleKeyword, scale.lower())
-    else:
-        scale = cast(ScaleKeyword, rcParams["stats.ic_scale"])
-    allowable = ["log", "negative_log", "deviance"] if NO_GET_ARGS else get_args(ScaleKeyword)
-    if scale not in allowable:
-        raise ValueError(f"{scale} is not a valid value for scale: must be in {allowable}")
-
-    if scale == "log":
-        scale_value = 1
-        ascending = False
-    else:
-        if scale == "negative_log":
-            scale_value = -1
-        else:
-            scale_value = -2
-        ascending = True
-
-    if ic is None:
-        ic = cast(ICKeyword, rcParams["stats.information_criterion"])
-    else:
-        ic = cast(ICKeyword, ic.lower())
-    allowable = ["loo", "waic"] if NO_GET_ARGS else get_args(ICKeyword)
-    if ic not in allowable:
-        raise ValueError(f"{ic} is not a valid value for ic: must be in {allowable}")
+    try:
+        (ics_dict, scale, ic) = _calculate_ics(compare_dict, scale=scale, ic=ic, var_name=var_name)
+    except Exception as e:
+        raise e.__class__("Encountered error in ic computation of compare.") from e
+    names = list(ics_dict.keys())
     if ic == "loo":
-        ic_func: Callable = loo
         df_comp = pd.DataFrame(
             index=names,
             columns=[
@@ -210,7 +190,6 @@ def compare(
         )
         scale_col = "loo_scale"
     elif ic == "waic":
-        ic_func = waic
         df_comp = pd.DataFrame(
             index=names,
             columns=[
@@ -230,6 +209,16 @@ def compare(
     else:
         raise NotImplementedError(f"The information criterion {ic} is not supported.")
 
+    if scale == "log":
+        scale_value = 1
+        ascending = False
+    else:
+        if scale == "negative_log":
+            scale_value = -1
+        else:
+            scale_value = -2
+        ascending = True
+
     method = rcParams["stats.ic_compare_method"] if method is None else method
     if method.lower() not in ["stacking", "bb-pseudo-bma", "pseudo-bma"]:
         raise ValueError(f"The method {method}, to compute weights, is not supported.")
@@ -238,17 +227,7 @@ def compare(
     p_ic = f"p_{ic}"
     ic_i = f"{ic}_i"
 
-    ics = pd.DataFrame()
-    names = []
-    for name, dataset in dataset_dict.items():
-        names.append(name)
-        try:
-            # Here is where the IC function is actually computed -- the rest of this
-            # function is argument processing and return value formatting
-            ics = ics.append([ic_func(dataset, pointwise=True, scale=scale, var_name=var_name)])
-        except Exception as e:
-            raise e.__class__(f"Encountered error trying to compute {ic} from model {name}.") from e
-    ics.index = names
+    ics = pd.DataFrame.from_dict(ics_dict, orient="index")
     ics.sort_values(by=ic, inplace=True, ascending=ascending)
     ics[ic_i] = ics[ic_i].apply(lambda x: x.values.flatten())
 
@@ -357,6 +336,135 @@ def _ic_matrix(ics, ic_i):
     return rows, cols, ic_i_val
 
 
+def _calculate_ics(
+    compare_dict,
+    scale: Optional[ScaleKeyword] = None,
+    ic: Optional[ICKeyword] = None,
+    var_name: Optional[str] = None,
+):
+    """Calculate loo and waic information criteria only if necessary.
+
+    It always calls the ic function with ``pointwise=True``.
+
+    Parameters
+    ----------
+    compare_dict :  dict of {str : InferenceData or ELPDData}
+        A dictionary of model names and InferenceData or ELPDData objects
+    scale : str, optional
+        Output scale for IC. Available options are:
+
+        - `log` : (default) log-score (after Vehtari et al. (2017))
+        - `negative_log` : -1 * (log-score)
+        - `deviance` : -2 * (log-score)
+
+        A higher log-score (or a lower deviance) indicates a model with better predictive accuracy.
+    ic : str, optional
+        Information Criterion (PSIS-LOO `loo` or WAIC `waic`) used to compare models.
+        Defaults to ``rcParams["stats.information_criterion"]``.
+    var_name : str, optional
+        Name of the variable storing pointwise log likelihood values in ``log_likelihood`` group.
+
+
+    Returns
+    -------
+    compare_dict : dict of ELPDData
+    scale : str
+    ic : str
+
+    """
+    precomputed_elpds = {
+        name: elpd_data
+        for name, elpd_data in compare_dict.items()
+        if isinstance(elpd_data, ELPDData)
+    }
+    precomputed_ic = None
+    precomputed_scale = None
+    if precomputed_elpds:
+        _, arbitrary_elpd = precomputed_elpds.popitem()
+        precomputed_ic = arbitrary_elpd.index[0]
+        precomputed_scale = arbitrary_elpd[f"{precomputed_ic}_scale"]
+        raise_non_pointwise = False
+        if not f"{precomputed_ic}_i" in arbitrary_elpd:
+            raise_non_pointwise = True
+        if precomputed_elpds:
+            if not all(
+                elpd_data.index[0] == precomputed_ic for elpd_data in precomputed_elpds.values()
+            ):
+                raise ValueError(
+                    "All information criteria to be compared must be the same "
+                    "but found both loo and waic."
+                )
+            if not all(
+                elpd_data[f"{precomputed_ic}_scale"] == precomputed_scale
+                for elpd_data in precomputed_elpds.values()
+            ):
+                raise ValueError("All information criteria to be compared must use the same scale")
+            if (
+                not all(
+                    f"{precomputed_ic}_i" in elpd_data for elpd_data in precomputed_elpds.values()
+                )
+                or raise_non_pointwise
+            ):
+                raise ValueError(
+                    "Not all provided ELPDData have been calculated with pointwise=True"
+                )
+        if ic is not None and ic.lower() != precomputed_ic:
+            warnings.warn(
+                "Provided ic argument is incompatible with precomputed elpd data. "
+                f"Using ic from precomputed elpddata: {precomputed_ic}"
+            )
+            ic = precomputed_ic
+        if scale is not None and scale.lower() != precomputed_scale:
+            warnings.warn(
+                "Provided scale argument is incompatible with precomputed elpd data. "
+                f"Using scale from precomputed elpddata: {precomputed_scale}"
+            )
+            scale = precomputed_scale
+
+    if ic is None and precomputed_ic is None:
+        ic = cast(ICKeyword, rcParams["stats.information_criterion"])
+    elif ic is None:
+        ic = precomputed_ic
+    else:
+        ic = cast(ICKeyword, ic.lower())
+    allowable = ["loo", "waic"] if NO_GET_ARGS else get_args(ICKeyword)
+    if ic not in allowable:
+        raise ValueError(f"{ic} is not a valid value for ic: must be in {allowable}")
+
+    if scale is None and precomputed_scale is None:
+        scale = cast(ScaleKeyword, rcParams["stats.ic_scale"])
+    elif scale is None:
+        scale = precomputed_scale
+    else:
+        scale = cast(ScaleKeyword, scale.lower())
+    allowable = ["log", "negative_log", "deviance"] if NO_GET_ARGS else get_args(ScaleKeyword)
+    if scale not in allowable:
+        raise ValueError(f"{scale} is not a valid value for scale: must be in {allowable}")
+
+    if ic == "loo":
+        ic_func: Callable = loo
+    elif ic == "waic":
+        ic_func = waic
+    else:
+        raise NotImplementedError(f"The information criterion {ic} is not supported.")
+
+    compare_dict = deepcopy(compare_dict)
+    for name, dataset in compare_dict.items():
+        if not isinstance(dataset, ELPDData):
+            try:
+                compare_dict[name] = ic_func(
+                    convert_to_inference_data(dataset),
+                    pointwise=True,
+                    scale=scale,
+                    var_name=var_name,
+                )
+            except Exception as e:
+                raise e.__class__(
+                    f"Encountered error trying to compute {ic} from model {name}."
+                ) from e
+    return (compare_dict, scale, ic)
+
+
 def hdi(
     ary,
     hdi_prob=None,
@@ -380,8 +488,8 @@ def hdi(
     ----------
     ary: obj
         object containing posterior samples.
-        Any object that can be converted to an az.InferenceData object.
-        Refer to documentation of az.convert_to_dataset for details.
+        Any object that can be converted to an :class:`arviz.InferenceData` object.
+        Refer to documentation of :func:`arviz.convert_to_dataset` for details.
     hdi_prob: float, optional
         Prob for which the highest density interval will be computed. Defaults to
         ``stats.hdi_prob`` rcParam.
@@ -398,14 +506,14 @@ def hdi(
         Specifies which InferenceData group should be used to calculate hdi.
         Defaults to 'posterior'
     var_names: list, optional
-        Names of variables to include in the hdi report. Prefix the variables by `~`
+        Names of variables to include in the hdi report. Prefix the variables by ``~``
         when you want to exclude them from the report: `["~beta"]` instead of `["beta"]`
-        (see `az.summary` for more details).
+        (see :func:`arviz.summary` for more details).
     filter_vars: {None, "like", "regex"}, optional, default=None
         If `None` (default), interpret var_names as the real variables names. If "like",
         interpret var_names as substrings of the real variables names. If "regex",
         interpret var_names as regular expressions on the real variables names. A la
-        `pandas.filter`.
+        ``pandas.filter``.
     coords: mapping, optional
         Specifies the subset over to calculate hdi.
     max_modes: int, optional
@@ -476,7 +584,7 @@ def hdi(
         "skipna": skipna,
         "out_shape": (max_modes, 2) if multimodal else (2,),
     }
-    kwargs.setdefault("output_core_dims", [["hdi", "mode"] if multimodal else ["hdi"]])
+    kwargs.setdefault("output_core_dims", [["mode", "hdi"] if multimodal else ["hdi"]])
     if not multimodal:
         func_kwargs["circular"] = circular
     else:
@@ -600,8 +708,9 @@ def loo(data, pointwise=None, var_name=None, reff=None, scale=None):
     Parameters
     ----------
     data: obj
-        Any object that can be converted to an az.InferenceData object. Refer to documentation of
-        az.convert_to_inference_data for details
+        Any object that can be converted to an :class:`arviz.InferenceData` object.
+        Refer to documentation of
+        :func:`arviz.convert_to_dataset` for details.
     pointwise: bool, optional
         If True the pointwise predictive accuracy will be returned. Defaults to
         ``stats.ic_pointwise`` rcParam.
@@ -609,21 +718,21 @@ def loo(data, pointwise=None, var_name=None, reff=None, scale=None):
         The name of the variable in log_likelihood groups storing the pointwise log
         likelihood data to use for loo computation.
     reff: float, optional
-        Relative MCMC efficiency, `ess / n` i.e. number of effective samples divided by the number
+        Relative MCMC efficiency, ``ess / n`` i.e. number of effective samples divided by the number
         of actual samples. Computed from trace by default.
     scale: str
         Output scale for loo. Available options are:
 
-        - `log` : (default) log-score
-        - `negative_log` : -1 * log-score
-        - `deviance` : -2 * log-score
+        - ``log`` : (default) log-score
+        - ``negative_log`` : -1 * log-score
+        - ``deviance`` : -2 * log-score
 
         A higher log-score (or a lower deviance or negative log_score) indicates a model with
         better predictive accuracy.
 
     Returns
     -------
-    ELPDData object (inherits from panda.Series) with the following row/attributes:
+    ELPDData object (inherits from :class:`pandas.Series`) with the following row/attributes:
     loo: approximated expected log pointwise predictive density (elpd)
     loo_se: standard error of loo
     p_loo: effective number of parameters
@@ -636,6 +745,14 @@ def loo(data, pointwise=None, var_name=None, reff=None, scale=None):
 
         The returned object has a custom print method that overrides pd.Series method.
 
+    See Also
+    --------
+    compare : Compare models based on PSIS-LOO loo or WAIC waic cross-validation.
+    waic : Compute the widely applicable information criterion.
+    plot_compare : Summary plot for model comparison.
+    plot_elpd : Plot pointwise elpd differences between two or more models.
+    plot_khat : Plot Pareto tail indices for diagnosing convergence.
+
     Examples
     --------
     Calculate LOO of a model:
@@ -774,7 +891,7 @@ def psislw(log_weights, reff=1.0):
     log_weights: array
         Array of size (n_observations, n_samples)
     reff: float
-        relative MCMC efficiency, `ess / n`
+        relative MCMC efficiency, ``ess / n``
 
     Returns
     -------
@@ -787,6 +904,10 @@ def psislw(log_weights, reff=1.0):
     ----------
     * Vehtari et al. (2015) see https://arxiv.org/abs/1507.02646
 
+    See Also
+    --------
+    loo : Compute Pareto-smoothed importance sampling leave-one-out cross-validation (PSIS-LOO-CV).
+
     Examples
     --------
     Get Pareto smoothed importance sampling (PSIS) log weights:
@@ -916,7 +1037,7 @@ def _gpdfit(ary):
     prior_bs = 3
     prior_k = 10
     n = len(ary)
-    m_est = 30 + int(n ** 0.5)
+    m_est = 30 + int(n**0.5)
 
     b_ary = 1 - np.sqrt(m_est / (np.arange(1, m_est + 1, dtype=float) - 0.5))
     b_ary /= prior_bs * ary[int(n / 4 + 0.5) - 1]
@@ -939,8 +1060,8 @@ def _gpdfit(ary):
     # estimate for k
     k_post = np.log1p(-b_post * ary).mean()  # pylint: disable=invalid-unary-operand-type,no-member
     # add prior for k_post
-    k_post = (n * k_post + prior_k * 0.5) / (n + prior_k)
     sigma = -k_post / b_post
+    k_post = (n * k_post + prior_k * 0.5) / (n + prior_k)
 
     return k_post, sigma
 
@@ -972,14 +1093,58 @@ def _gpinv(probs, kappa, sigma):
     return x
 
 
+def r2_samples(y_true, y_pred):
+    """R² samples for Bayesian regression models. Only valid for linear models.
+
+    Parameters
+    ----------
+    y_true: array-like of shape = (n_outputs,)
+        Ground truth (correct) target values.
+    y_pred: array-like of shape = (n_posterior_samples, n_outputs)
+        Estimated target values.
+
+    Returns
+    -------
+    Pandas Series with the following indices:
+    Bayesian R² samples.
+
+    See Also
+    --------
+    plot_lm : Posterior predictive and mean plots for regression-like data.
+
+    Examples
+    --------
+    Calculate R² samples for Bayesian regression models :
+
+    .. ipython::
+
+        In [1]: import arviz as az
+           ...: data = az.load_arviz_data('regression1d')
+           ...: y_true = data.observed_data["y"].values
+           ...: y_pred = data.posterior_predictive.stack(sample=("chain", "draw"))["y"].values.T
+           ...: az.r2_samples(y_true, y_pred)
+
+    """
+    _numba_flag = Numba.numba_flag
+    if y_pred.ndim == 1:
+        var_y_est = _numba_var(svar, np.var, y_pred)
+        var_e = _numba_var(svar, np.var, (y_true - y_pred))
+    else:
+        var_y_est = _numba_var(svar, np.var, y_pred, axis=1)
+        var_e = _numba_var(svar, np.var, (y_true - y_pred), axis=1)
+    r_squared = var_y_est / (var_y_est + var_e)
+
+    return r_squared
+
+
 def r2_score(y_true, y_pred):
     """R² for Bayesian regression models. Only valid for linear models.
 
     Parameters
     ----------
-    y_true: array-like of shape = (n_samples) or (n_samples, n_outputs)
+    y_true: array-like of shape = (n_outputs,)
         Ground truth (correct) target values.
-    y_pred: array-like of shape = (n_samples) or (n_samples, n_outputs)
+    y_pred: array-like of shape = (n_posterior_samples, n_outputs)
         Estimated target values.
 
     Returns
@@ -988,6 +1153,10 @@ def r2_score(y_true, y_pred):
     r2: Bayesian R²
     r2_std: standard deviation of the Bayesian R².
 
+    See Also
+    --------
+    plot_lm : Posterior predictive and mean plots for regression-like data.
+
     Examples
     --------
     Calculate R² for Bayesian regression models :
@@ -1001,15 +1170,7 @@ def r2_score(y_true, y_pred):
            ...: az.r2_score(y_true, y_pred)
 
     """
-    _numba_flag = Numba.numba_flag
-    if y_pred.ndim == 1:
-        var_y_est = _numba_var(svar, np.var, y_pred)
-        var_e = _numba_var(svar, np.var, (y_true - y_pred))
-    else:
-        var_y_est = _numba_var(svar, np.var, y_pred.mean(0))
-        var_e = _numba_var(svar, np.var, (y_true - y_pred), axis=0)
-    r_squared = var_y_est / (var_y_est + var_e)
-
+    r_squared = r2_samples(y_true=y_true, y_pred=y_pred)
     return pd.Series([np.mean(r_squared), np.std(r_squared)], index=["r2", "r2_std"])
 
 
@@ -1036,17 +1197,17 @@ def summary(
     Parameters
     ----------
     data: obj
-        Any object that can be converted to an az.InferenceData object
-        Refer to documentation of az.convert_to_dataset for details
+        Any object that can be converted to an :class:`arviz.InferenceData` object
+        Refer to documentation of :func:`arviz.convert_to_dataset` for details
     var_names: list
-        Names of variables to include in summary. Prefix the variables by `~` when you
+        Names of variables to include in summary. Prefix the variables by ``~`` when you
         want to exclude them from the summary: `["~beta"]` instead of `["beta"]` (see
         examples below).
     filter_vars: {None, "like", "regex"}, optional, default=None
         If `None` (default), interpret var_names as the real variables names. If "like",
         interpret var_names as substrings of the real variables names. If "regex",
         interpret var_names as regular expressions on the real variables names. A la
-        `pandas.filter`.
+        ``pandas.filter``.
     coords: Dict[str, List[Any]], optional
         Coordinate subset for which to calculate the summary.
     group: str
@@ -1069,7 +1230,7 @@ def summary(
 
         The functions will be given one argument, the samples for a variable as an nD array,
         The functions should be in the style of a ufunc and return a single number. For example,
-        `np.mean`, or `scipy.stats.var` would both work.
+        :func:`numpy.mean`, or ``scipy.stats.var`` would both work.
     extend: boolean
         If True, use the statistics returned by ``stat_funcs`` in addition to, rather than in place
         of, the default statistics. This is only meaningful when ``stat_funcs`` is not None.
@@ -1099,6 +1260,15 @@ def summary(
         `r_hat`.
         `r_hat` is only computed for traces with 2 or more chains.
 
+    See Also
+    --------
+    waic : Compute the widely applicable information criterion.
+    loo : Compute Pareto-smoothed importance sampling leave-one-out
+          cross-validation (PSIS-LOO-CV).
+    ess : Calculate estimate of the effective sample size (ess).
+    rhat : Compute estimate of rank normalized splitR-hat for a set of traces.
+    mcse : Calculate Markov Chain Standard Error statistic.
+
     Examples
     --------
     .. ipython::
@@ -1107,15 +1277,15 @@ def summary(
            ...: data = az.load_arviz_data("centered_eight")
            ...: az.summary(data, var_names=["mu", "tau"])
 
-    You can use `filter_vars` to select variables without having to specify all the exact
-    names. Use `filter_vars="like"` to select based on partial naming:
+    You can use ``filter_vars`` to select variables without having to specify all the exact
+    names. Use ``filter_vars="like"`` to select based on partial naming:
 
     .. ipython::
 
         In [1]: az.summary(data, var_names=["the"], filter_vars="like")
 
-    Use `filter_vars="regex"` to select based on regular expressions, and prefix the variables
-    you want to exclude by `~`. Here, we exclude from the summary all the variables
+    Use ``filter_vars="regex"`` to select based on regular expressions, and prefix the variables
+    you want to exclude by ``~``. Here, we exclude from the summary all the variables
     starting with the letter t:
 
     .. ipython::
@@ -1372,8 +1542,8 @@ def waic(data, pointwise=None, var_name=None, scale=None, dask_kwargs=None):
     Parameters
     ----------
     data: obj
-        Any object that can be converted to an az.InferenceData object. Refer to documentation of
-        ``az.convert_to_inference_data`` for details
+        Any object that can be converted to an :class:`arviz.InferenceData` object.
+        Refer to documentation of :func:`arviz.convert_to_inference_data` for details.
     pointwise: bool
         If True the pointwise predictive accuracy will be returned. Defaults to
         ``stats.ic_pointwise`` rcParam.
@@ -1394,17 +1564,24 @@ def waic(data, pointwise=None, var_name=None, scale=None, dask_kwargs=None):
 
     Returns
     -------
-    ELPDData object (inherits from panda.Series) with the following row/attributes:
+    ELPDData object (inherits from :class:`pandas.Series`) with the following row/attributes:
     waic: approximated expected log pointwise predictive density (elpd)
     waic_se: standard error of waic
     p_waic: effective number parameters
     var_warn: bool
         True if posterior variance of the log predictive densities exceeds 0.4
-    waic_i: xarray.DataArray with the pointwise predictive accuracy, only if pointwise=True
+    waic_i: :class:`~xarray.DataArray` with the pointwise predictive accuracy,
+            only if pointwise=True
     waic_scale: scale of the reported waic results
 
         The returned object has a custom print method that overrides pd.Series method.
 
+    See Also
+    --------
+    loo : Compute Pareto-smoothed importance sampling leave-one-out cross-validation (PSIS-LOO-CV).
+    compare : Compare models based on PSIS-LOO-CV or WAIC.
+    plot_compare : Summary plot for model comparison.
+
     Examples
     --------
     Calculate WAIC of a model:
@@ -1519,13 +1696,13 @@ def loo_pit(idata=None, *, y=None, y_hat=None, log_weights=None):
     Parameters
     ----------
     idata: InferenceData
-        InferenceData object.
+        :class:`arviz.InferenceData` object.
     y: array, DataArray or str
-        Observed data. If str, idata must be present and contain the observed data group
+        Observed data. If str, ``idata`` must be present and contain the observed data group
     y_hat: array, DataArray or str
         Posterior predictive samples for ``y``. It must have the same shape as y plus an
         extra dimension at the end of size n_samples (chains and draws stacked). If str or
-        None, idata must contain the posterior predictive group. If None, y_hat is taken
+        None, ``idata`` must contain the posterior predictive group. If None, y_hat is taken
         equal to y, thus, y must be str too.
     log_weights: array or DataArray
         Smoothed log_weights. It must have the same shape as ``y_hat``
@@ -1537,6 +1714,14 @@ def loo_pit(idata=None, *, y=None, y_hat=None, log_weights=None):
     loo_pit: array or DataArray
         Value of the LOO-PIT at each observed data point.
 
+    See Also
+    --------
+    plot_loo_pit : Plot Leave-One-Out probability integral transformation (PIT) predictive checks.
+    loo : Compute Pareto-smoothed importance sampling leave-one-out
+          cross-validation (PSIS-LOO-CV).
+    plot_elpd : Plot pointwise elpd differences between two or more models.
+    plot_khat : Plot Pareto tail indices for diagnosing convergence.
+
     Examples
     --------
     Calculate LOO-PIT values using as test quantity the observed values themselves.
@@ -1612,8 +1797,8 @@ def loo_pit(idata=None, *, y=None, y_hat=None, log_weights=None):
 
     if len(y.shape) + 1 != len(y_hat.shape):
         raise ValueError(
-            f"y_hat must have 1 more dimension than y, but y_hat has {len(y.shape)} dims and y has "
-            f"{len(y_hat.shape)} dims"
+            f"y_hat must have 1 more dimension than y, but y_hat has {len(y_hat.shape)} dims and "
+            f"y has {len(y.shape)} dims"
         )
 
     if y.shape != y_hat.shape[:-1]:
@@ -1681,9 +1866,9 @@ def apply_test_function(
     Parameters
     ----------
     idata: InferenceData
-        InferenceData object on which to apply the test function. This function will add
-        new variables to the InferenceData object to store the result without modifying the
-        existing ones.
+        :class:`arviz.InferenceData` object on which to apply the test function.
+        This function will add new variables to the InferenceData object
+        to store the result without modifying the existing ones.
     func: callable
         Callable that calculates the test function. It must have the following call signature
         ``func(y, theta, *args, **kwargs)`` (where ``y`` is the observed data or posterior
@@ -1727,6 +1912,10 @@ def apply_test_function(
         Output InferenceData object. If ``inplace=True``, it is the same input object modified
         inplace.
 
+    See Also
+    --------
+    plot_bpv :  Plot Bayesian p-value for observed data and Posterior/Prior predictive.
+
     Notes
     -----
     This function is provided for convenience to wrap scalar or functions working on low
@@ -1735,7 +1924,7 @@ def apply_test_function(
 
     Examples
     --------
-    Use ``apply_test_function`` to wrap ``np.min`` for illustration purposes. And plot the
+    Use ``apply_test_function`` to wrap ``numpy.min`` for illustration purposes. And plot the
     results.
 
     .. plot::
