@@ -3,13 +3,15 @@ from sys import version_info
 from typing import Dict, Tuple
 
 import numpy as np
+import pkg_resources
 import packaging
 import pandas as pd
 import pytest
 from numpy import ma
 
-from arviz import (  # pylint: disable=wrong-import-position
+from ... import (  # pylint: disable=wrong-import-position
     InferenceData,
+    from_dict,
     from_pymc3,
     from_pymc3_predictions,
 )
@@ -23,8 +25,20 @@ from ..helpers import (  # pylint: disable=unused-import, wrong-import-position
     load_cached_models,
 )
 
-# Skip all tests if pymc3 not installed
-pm = importorskip("pymc3")
+# Skip all tests unless running on pymc3 v3
+try:
+    pymc3_version = pkg_resources.get_distribution("pymc3").version
+    PYMC3_V4 = packaging.version.parse(pymc3_version) >= packaging.version.parse("4.0")
+    PYMC3_installed = True
+    import pymc3 as pm
+except pkg_resources.DistributionNotFound:
+    PYMC3_V4 = False
+    PYMC3_installed = False
+
+pytestmark = pytest.mark.skipif(
+    not PYMC3_installed or PYMC3_V4,
+    reason="Run tests only if pymc3 installed and its version is <4.0",
+)
 
 
 class TestDataPyMC3:
@@ -107,7 +121,7 @@ class TestDataPyMC3:
                 )
 
     def test_from_pymc_predictions(self, data, eight_schools_params):
-        "Test that we can add predictions to a previously-existing InferenceData."
+        """Test that we can add predictions to a previously-existing InferenceData."""
         test_dict = {
             "posterior": ["mu", "tau", "eta", "theta"],
             "sample_stats": ["diverging", "lp"],
@@ -139,6 +153,15 @@ class TestDataPyMC3:
             assert ivalues.shape[0] == 1  # one chain in predictions
             assert np.all(np.isclose(ivalues[0], values))
 
+    def test_from_pymc_trace_inference_data(self):
+        """Check if the error is raised successfully after passing InferenceData as trace"""
+        idata = from_dict(
+            posterior={"A": np.random.randn(2, 10, 2), "B": np.random.randn(2, 10, 5, 2)}
+        )
+        assert isinstance(idata, InferenceData)
+        with pytest.raises(ValueError):
+            from_pymc3(trace=idata, model=pm.Model())
+
     def test_from_pymc_predictions_new(self, data, eight_schools_params):
         # check creating new
         inference_data, posterior_predictive = self.make_predictions_inference_data(
@@ -147,7 +170,7 @@ class TestDataPyMC3:
         test_dict = {
             "posterior": ["mu", "tau", "eta", "theta"],
             "predictions": ["obs"],
-            "~observed_data": "",
+            "~observed_data": [""],
         }
         fails = check_multiple_attrs(test_dict, inference_data)
         assert not fails
@@ -162,7 +185,7 @@ class TestDataPyMC3:
             elif len(ivalues.shape) == 2:
                 ivalues_arr = np.reshape(ivalues.values, (ivalues.shape[0] * ivalues.shape[1]))
             else:
-                raise ValueError("Unexpected values shape for variable %s" % key)
+                raise ValueError(f"Unexpected values shape for variable {key}")
             assert (ivalues.shape[0] == 2) and (ivalues.shape[1] == 500)
             assert values.shape[0] == 1000
             assert np.all(np.isclose(ivalues_arr, values))
@@ -329,7 +352,7 @@ class TestDataPyMC3:
         }
         if not log_likelihood:
             test_dict.pop("log_likelihood")
-            test_dict["~log_likelihood"] = []
+            test_dict["~log_likelihood"] = [""]
         if isinstance(log_likelihood, list):
             test_dict["log_likelihood"] = ["y1", "~y2"]
 
@@ -453,7 +476,7 @@ class TestDataPyMC3:
             # assert predictive_trace["obs"].shape == (400, 2)
             # but the shape seems to vary between pymc3 versions
             inference_data = from_pymc3_predictions(predictive_trace, posterior_trace=trace)
-        test_dict = {"posterior": ["beta"], "~observed_data": ""}
+        test_dict = {"posterior": ["beta"], "~observed_data": [""]}
         fails = check_multiple_attrs(test_dict, inference_data)
         assert not fails, "Posterior data not copied over as expected."
         test_dict = {"predictions": ["obs"]}
@@ -511,7 +534,7 @@ class TestDataPyMC3:
             "prior_predictive": ["obs"],
         }
         if use_context:
-            with model:
+            with model:  # pylint: disable=not-context-manager
                 inference_data = from_pymc3(prior=prior)
         else:
             inference_data = from_pymc3(prior=prior, model=model)
@@ -530,7 +553,7 @@ class TestDataPyMC3:
             inference_data = from_pymc3(prior=prior)
         test_dict = {
             "prior": ["beta", "obs"],
-            "~prior_predictive": [],
+            "~prior_predictive": [""],
         }
         fails = check_multiple_attrs(test_dict, inference_data)
         assert not fails
@@ -583,8 +606,8 @@ class TestPyMC3WarmupHandling:
             f"{post_prefix}sample_stats": ["~tune", "accept"],
             f"{warmup_prefix}warmup_posterior": ["u1", "n1"],
             f"{warmup_prefix}warmup_sample_stats": ["~tune"],
-            "~warmup_log_likelihood": [],
-            "~log_likelihood": [],
+            "~warmup_log_likelihood": [""],
+            "~log_likelihood": [""],
         }
         fails = check_multiple_attrs(test_dict, idata)
         assert not fails
@@ -621,8 +644,8 @@ class TestPyMC3WarmupHandling:
         test_dict = {
             "posterior": ["u1", "n1"],
             "sample_stats": ["~tune", "accept"],
-            "~warmup_posterior": [],
-            "~warmup_sample_stats": [],
+            "~warmup_posterior": [""],
+            "~warmup_sample_stats": [""],
         }
         fails = check_multiple_attrs(test_dict, idata)
         assert not fails
@@ -667,8 +690,8 @@ class TestPyMC3WarmupHandling:
             test_dict = {
                 "posterior": ["u1", "n1"],
                 "sample_stats": ["~tune", "accept"],
-                "~warmup_posterior": [],
-                "~warmup_sample_stats": [],
+                "~warmup_posterior": [""],
+                "~warmup_sample_stats": [""],
             }
             fails = check_multiple_attrs(test_dict, idata)
             assert not fails
